#!/usr/bin/env python
"""
@file    networkStatistics.py
@author  Yun-Pang Wang
@author  Michael Behrisch
@date    2007-02-27
@version $Id: networkStatistics.py 14677 2013-09-11 08:30:08Z behrisch $

This script is to calculate the global performance indices according to the SUMO-based simulation results.
Besides, this script is also to execute the significance test for evaluating the results from different assignment methods.
The t test and the Kruskal-Wallis test are available in this script. 
If not specified, the Kruskal-Wallis test will be applied with the assumption that data are not normally distributed.

The analyzed parameters include:
- travel length
- travel time
- travel speed
- stop time

SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
Copyright (C) 2008-2013 DLR (http://www.dlr.de/) and contributors

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""

import os, sys, datetime, math, operator

from xml.sax import saxutils, make_parser, handler
from optparse import OptionParser
from statisticsElements import Assign, T_Value, H_Value, VehInformationReader, getStatisticsOutput, getSignificanceTestOutput
from tables import chiSquareTable, tTable


def getBasicStats(verbose, method, vehicles, assignments):
    totalVeh = 0.
    totalTravelTime = 0.
    totalTravelLength = 0.
    totalTravelSpeed = 0.
    totalWaitTime = 0.
    totalDiffTime = 0.
    totalDiffSpeed = 0.
    totalDiffLength = 0.
    totalDiffWaitTime = 0.
    totalDiffTravelTime = 0.
    totalDepartDelay = 0.
          
    for veh in vehicles:
        totalVeh += 1
        veh.method = method
        # unit: speed - m/s; traveltime - s; travel length - m    
        veh.speed = veh.travellength / veh.traveltime
        totalTravelTime += veh.traveltime
        totalTravelLength += veh.travellength
        totalWaitTime += veh.waittime
        totalTravelSpeed += veh.speed
        totalDepartDelay += veh.departdelay  
    if verbose:    
        print 'totalVeh:', totalVeh
    avgTravelTime = totalTravelTime/totalVeh
    avgTravelLength = totalTravelLength/totalVeh
    avgTravelSpeed = totalTravelSpeed/totalVeh
    avgWaitTime = totalWaitTime/totalVeh
    avgDepartDelay = totalDepartDelay/totalVeh
    for veh in vehicles:
        totalDiffTravelTime += (veh.traveltime - avgTravelTime)**2
        totalDiffSpeed += (veh.speed - avgTravelSpeed)**2
        totalDiffLength += (veh.travellength - avgTravelLength)**2
        totalDiffWaitTime += (veh.waittime - avgWaitTime)**2
    
    #SD: standard deviation
    SDTravelTime = (totalDiffTravelTime/totalVeh)**(0.5)
    SDLength = (totalDiffLength/totalVeh)**(0.5)
    SDSpeed = (totalDiffSpeed/totalVeh)**(0.5)
    SDWaitTime = (totalDiffWaitTime/totalVeh)**(0.5)

    assignments[method] = Assign(method, totalVeh, totalTravelTime, totalTravelLength,
                                 totalDepartDelay, totalWaitTime, avgTravelTime,
                                 avgTravelLength, avgTravelSpeed, avgDepartDelay,
                                 avgWaitTime, SDTravelTime, SDLength, SDSpeed, SDWaitTime)
                     

# The observations should be drawn from a normally distributed population. 
# For two independent samples, the t test has the additional requirement
# that the population standard deviations should be equal.
def doTTestForAvg(verbose, tValueAvg, assignments):
    if verbose:
       print 'begin the t test!'
    for num, A in enumerate(assignments):
        for B in assignments[num+1: ]:
            sdABTravelTime = 0.
            sdABSpeed = 0.
            sdABLength = 0.
            sdABWaitTime = 0.      
            if str(A.label) != str(B.label):
                sdABTravelTime = (float(A.totalVeh-1)*((A.SDTravelTime)**2) + float(B.totalVeh -1)*((B.SDTravelTime)**2))/float(A.totalVeh + B.totalVeh-2)
                sdABTravelTime = sdABTravelTime**(0.5)
                
                sdABSpeed = (float(A.totalVeh-1)*((A.SDSpeed)**2) + float(B.totalVeh -1)*((B.SDSpeed)**2))/float(A.totalVeh + B.totalVeh-2)
                sdABSpeed = sdABSpeed**(0.5)
                
                sdABLength = (float(A.totalVeh-1)*((A.SDLength)**2) + float(B.totalVeh -1)*((B.SDLength)**2))/float(A.totalVeh + B.totalVeh-2)
                sdABLength = sdABLength**(0.5)
                
                sdABWaitTime = (float(A.totalVeh-1)*((A.SDWaitTime)**2) + float(B.totalVeh -1)*((B.SDWaitTime)**2))/float(A.totalVeh + B.totalVeh-2)
                sdABWaitTime = sdABWaitTime**(0.5)

                tempvalue = (float(A.totalVeh * B.totalVeh)/float(A.totalVeh + B.totalVeh))**0.5
                
                avgtraveltime = abs(A.avgTravelTime - B.avgTravelTime)/sdABTravelTime * tempvalue
                
                avgtravelspeed = abs(A.avgTravelSpeed - B.avgTravelSpeed)/sdABSpeed * tempvalue
                
                avgtravellength = abs(A.avgTravelLength - B.avgTravelLength)/sdABLength * tempvalue
                
                if sdABWaitTime != 0.:
                    avgwaittime = abs(A.avgWaitTime - B.avgWaitTime)/sdABWaitTime * tempvalue
                else:
                    avgwaittime = 0.
                    print 'check if the information about veh.waittime exists!'
                freedomdegree = A.totalVeh + B.totalVeh - 2
                if freedomdegree > 30 and freedomdegree <= 40:
                    freedomdegree = 31
                if freedomdegree > 40 and freedomdegree <= 50:
                    freedomdegree = 32
                if freedomdegree > 50 and freedomdegree <= 60:
                    freedomdegree = 33
                if freedomdegree > 60 and freedomdegree <= 80:
                    freedomdegree = 34
                if freedomdegree > 80 and freedomdegree <= 100:
                    freedomdegree = 35
                if freedomdegree > 100:
                    freedomdegree = 36
                lowvalue = tTable[freedomdegree][6]
                highvalue = tTable[freedomdegree][9]
                
                tValueAvg[A][B] = T_Value(avgtraveltime, avgtravelspeed, avgtravellength, avgwaittime, lowvalue, highvalue)
                     

def doKruskalWallisTest(verbose, groups, combivehlist, assignments, label, hValues):
    if verbose:
        print '\nbegin the Kruskal-Wallis test!'
        print 'methods:', label
        print 'number of samples:', len(combivehlist)
    adjlabel = label + '_' + "adjusted"
    if groups >= 100.:
        groups = 100.
    lowvalue = chiSquareTable[int(groups)-1][2]
    highvalue = chiSquareTable[int(groups)-1][4]
   
    H = H_Value(label, lowvalue, highvalue)
    adjH = H_Value(adjlabel, lowvalue, highvalue)
    hValues.append(H)
    hValues.append(adjH)
    
    for index in [("traveltime"),("speed"),("travellength"),("waittime")]:
        for veh in combivehlist:
            veh.rank = 0.
        for method in assignments.itervalues():
            method.sumrank = 0.
            
        samecountlist = []
        current = 0
        lastrank = 0
        subtotal = 0.
        adjusted = 0.
        
        combivehlist.sort(key=operator.attrgetter(index))
        totalsamples = len(combivehlist)

        for i in range (0,len(combivehlist)):
            samecount = 0
            if i <= current:
                if index == "traveltime":
                    value = combivehlist[current].traveltime
                elif index == "speed":
                    value = combivehlist[current].speed
                elif index == "travellength":
                    value = combivehlist[current].travellength
                elif index == "waittime":
                    value = combivehlist[current].waittime
            else:
                print 'error!'

            for j in range (current,len(combivehlist)):
                if index == "traveltime":
                    if combivehlist[j].traveltime == value:
                        samecount += 1
                    else:
                        break
                elif index == "speed":
                    if combivehlist[j].speed == value:
                        samecount += 1
                    else:
                        break
                elif index == "travellength":
                    if combivehlist[j].travellength == value:
                        samecount += 1
                    else:
                        break
                elif index == "waittime":
                    if combivehlist[j].waittime == value:
                        samecount += 1
                    else:
                        break                    

            if samecount == 1.:
                lastrank += 1.
                combivehlist[current].rank = lastrank
            else:
                sumrank = 0.
                for j in range (0, samecount):
                    lastrank += 1.
                    sumrank += lastrank
                rank = sumrank/samecount
                for j in range (0, samecount):
                    combivehlist[current+j].rank = rank
                
                elem = (value, samecount)
                samecountlist.append(elem)
            
            current = current + samecount
            
            if current > (len(combivehlist) - 1):
                break
                print 'current:', current
                            
        for veh in combivehlist:
            for method in assignments.itervalues():
                if veh.method == method.label:
                    method.sumrank += veh.rank

        for method in assignments.itervalues():
            subtotal += (method.sumrank**2.) / method.totalVeh
            
        for elem in samecountlist:
            adjusted += (float(elem[1]**3) - float(elem[1]))
        
        c = 1. - (adjusted /float(totalsamples**3 - totalsamples))            

        if index == "traveltime":
            H.traveltime = 12./(totalsamples*(totalsamples+1)) * subtotal - 3.*(totalsamples+1)
            if c > 0.:
                adjH.traveltime = H.traveltime / c
        elif index == "speed":
            H.travelspeed = 12./(totalsamples*(totalsamples+1)) * subtotal - 3.*(totalsamples+1)
            if c > 0.:
                adjH.travelspeed = H.travelspeed / c
        elif index == "travellength":
            H.travellength = 12./(totalsamples*(totalsamples+1)) * subtotal - 3.*(totalsamples+1)
            if c > 0.:
                adjH.travellength = H.travellength / c
        elif index == "waittime":    
            H.waittime = 12./(totalsamples*(totalsamples+1)) * subtotal - 3.*(totalsamples+1)
            if c > 0.:
                adjH.waittime = H.waittime / c
                
optParser = OptionParser()
optParser.add_option("-t", "--tripinform-file", dest="vehfile",
                     help="read vehicle information generated by the DUA assignment from FILE (mandatory)", metavar="FILE")
optParser.add_option("-o", "--output-file", dest="outputfile", default="Global_MOE.txt",
                     help="write output to FILE", metavar="FILE")
optParser.add_option("-g", "--SGToutput-file", dest="sgtoutputfile", default="significanceTest.txt",
                     help="write output to FILE", metavar="FILE")
optParser.add_option("-v", "--verbose", action="store_true", dest="verbose",
                     default=False, help="tell me what you are doing")
optParser.add_option("-e", "--tTest", action="store_true", dest="ttest",
                     default=False, help="perform the t-Test")
optParser.add_option("-k", "--kruskalWallisTest", action="store_true", dest="kwtest",
                     default=False, help="perform the Kruskal-Wallis-Test")
                     
                                       
(options, args) = optParser.parse_args()

if not options.vehfile:
    optParser.print_help()
    sys.exit()
parser = make_parser()

allvehicles = {}
for filename in options.vehfile.split(","):
    allvehicles[filename] = []
    parser.setContentHandler(VehInformationReader(allvehicles[filename]))
    parser.parse(filename)

# Vehicles from dua, incremental, clogit and oneshot are in included in the allvehlist.

# The results of the t test are stored in the tValueAvg.
tValueAvg = {}
# The resultes of the Kruskal-Wallis test are stored in the hValues.
hValues = []

# intitalization
combilabel = ''

assignments = {}
# calculate/read the basic statistics
for method, vehicles in allvehicles.iteritems():
    getBasicStats(options.verbose, method, vehicles, assignments)

getStatisticsOutput(assignments, options.outputfile)
print 'The calculation of network statistics is done!'              

# begin the significance test for the observations with a normal distribution
if options.ttest:
    print 'begin the t test!'
    for A in assignments.itervalues():
        tValueAvg[A] = {}
    doTTestForAvg(options.verbose, tValueAvg, list(assignments.itervalues()))
    print 'The t test is done!'        
if options.kwtest:
    # The Kruskal-Wallis test is applied for the data, not drawn from a normally distributed population.
    groups = 2
    values = list(allvehicles.iteritems())
    for num, A in enumerate(values):
        for B in values[num+1: ]:
            combilabel = ''
            combivehlist = []
            combilabel = A[0] + '_' + B[0]
            print 'Test for:', combilabel
            for veh in A[1]:
                combivehlist.append(veh)
            for veh in B[1]:
                combivehlist.append(veh)
            doKruskalWallisTest(options.verbose, groups, combivehlist, assignments, combilabel, hValues)
        
getSignificanceTestOutput(assignments, options.ttest, tValueAvg, hValues, options.sgtoutputfile)
print 'The Significance test is done!'



# -*- coding: Latin-1 -*-
"""
@file    FCDvsCompleteRoute.py
@author  Sascha Krieg
@author  Daniel Krajzewicz
@author  Michael Behrisch
@date    2008-04-08

Compares the FCD-route of an taxi with the generated route which is used in the simulation.
The output data can be visualized with the script My_mpl_dump_onNet based on mpl_dump_onNet from Daniel.
(avg=False)

Secondly the average of added Edges per route will be calculate (avg=True). 

SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
Copyright (C) 2008-2013 DLR (http://www.dlr.de/) and contributors

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""

import util.Path as path

#global vars

edgeList=[]
routeDict= {}
taxi="351_11" #the Taxi for which the output should be generated
avg=False

def main():
    print "start program"    
    readRoutes()
    if avg: 
        clacAvg()
    else:   
        writeOutput()
    print "end"
    

def readRoutes():
    """reads the t1CompletePath and the taxiRoutesPath. 
        and sets for each edge in t1CompletePath the color=green
        and if the edge is also in taxiRoutesPath changes the color to red.
    """  
       
    def countEdges(line): 
        #get Taxi-Id
        id=line.split('"')[1]
        line=inputFile.next() #go to next line with edges
         
        words=line[line.find(">")+1:line.find("</")].split(" ")
        lastEdge=words[0]
        no=0
        for edge in words[1:]:
            if edge.find('/')!=-1: #ignore edges with are used only in SUMO
                continue   
               
            if lastEdge!=edge:                                              
                no+=1
            lastEdge=edge
        #no=len(line[line.find(">")+1:line.find("</")].split(" ")) #splited in single edges
        routeDict.setdefault(id,[]).append(no)        
            
    inputFile=open(path.taxiRoutesComplete,'r')
    for line in inputFile:
        if line.find("<vehicle id=\""+taxi+"\"")!=-1 and not avg:
            line=inputFile.next()
            #         delete edges tag at start and end    
            words=line[line.find(">")+1:line.find("</")].split(" ")
            for edge in words:
                edgeList.append("id=\""+edge+"\" no=\"0.9\"")
            print edgeList
            
        #for calc of avg    
        if line.find("<vehicle id=")!=-1 and avg:
            countEdges(line) 
    inputFile.close()
    
    #read taxiRoutesPath
    inputFile=open(path.taxiRoutes,'r')
    for line in inputFile:
        if line.find("<vehicle id=\""+taxi+"\"")!=-1 and not avg:
            line=inputFile.next()
            #         delete edges tag at start and end    
            words=line[line.find(">")+1:line.find("</")].split(" ")
            for edge in words:                
                if "id=\""+edge+"\" no=\"0.9\"" in edgeList:
                    edgeList[edgeList.index("id=\""+edge+"\" no=\"0.9\"")]="id=\""+edge+"\" no=\"0.1\""
                elif "id=\""+edge+"\" no=\"0.1\"" not in edgeList:
                    # if edge only is in the taxiRoutesPath file (FCD) color =yellow
                    edgeList.append("id=\""+edge+"\" no=\"0.5\"")
            print edgeList
            
        #for calc of avg    
        if line.find("<vehicle id=")!=-1 and avg:
            countEdges(line) 
    inputFile.close()

def clacAvg():
    diffList=[]
    orgList=[]
    compList=[]
    ids=set()
    diffList200=[]
    orgList200=[]
   
    for id,noList in routeDict.iteritems(): 
        if len(noList)<2:
            continue       
        orgList.append(noList[1])  
        compList.append(noList[0])  
        diffList.append(noList[0]-noList[1])
        if noList[0]-noList[1]>0:
            #print id,noList 
            ids.add(id)
            diffList200.append(noList[0]-noList[1])
            orgList200.append(noList[1])  
        #In Prozent
        # Kanten die nur in Sumo sind mit "/" bei berechnung entfernen
    # Zeige Routen bei denen etwas hinzugfügt wurde    
        
    
   
    print "len Routen gesamt", len(diffList)
    print "avg (Absolut) Kanten hinzugefügt", sum(diffList)/(len(diffList)+0.0)
    print "Relavg", sum(diffList)/(len(diffList)+0.0)/(sum(orgList)/(len(orgList)+0.0))*100, "%"
    print "avgOrg", sum(orgList)/(len(orgList)+0.0)," edges"
    print "avgCompleted", sum(compList)/(len(compList)+0.0)," edges"
   
    print
    print "Betrachtung der hinzugefügten Kanten nur für die Taxis bei denen Tatsächlich auch Kanten hinzugefügt wurden"
    print "ids (Taxis/Routen)", len(ids)
    print "% der Gesamtrouten", 100.0*len(ids)/len(diffList)
    print "avg 200 (ids)", sum(diffList200)/(len(diffList200)+0.0)
    print "Relavg 200", sum(diffList200)/(len(diffList200)+0.0)/(sum(orgList200)/(len(orgList200)+0.0))*100, "%"
    
    
def writeOutput():
    """Writes an XML-File with the extracted results"""
    outputFile=open(path.fcdVsCompleteRoute+str(taxi)+".out.xml",'w')
    outputFile.write("<netstats>\n")
    outputFile.write("\t<interval begin=\"0\" end=\"899\" id=\"dump_900\">\n")
    for k in edgeList:         
        outputFile.write("\t\t<edge "+k+" color=\"1.0\"/>\n")
    outputFile.write("\t</interval>\n")        
    outputFile.write("</netstats>")

    
#start the program
main()


# -*- coding: Latin-1 -*-
"""
@file    FetchData.py
@author  Sascha Krieg
@author  Daniel Krajzewicz
@author  Michael Behrisch
@date    2008-04-15

Fetches Information of the simulated Data and the FCD.
For each taxi the time, speed and position of every step will be collected and stored in a file. 

SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
Copyright (C) 2008-2013 DLR (http://www.dlr.de/) and contributors

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""

import profile
import util.Path as path
from util.CalcTime import getTimeInSecs
from fcdToRoutes.GenerateTaxiRoutes import readFCDComplete,readSimFCDComplete,getSimTaxiId  
from util.Reader import readRoute_EdgesMod

#global vars
rawFcdDict={}
fcdDict={}
simRawFcdDict={}
simFcdDict={}
vtypeDict={}
withoutEmptyEdges=True # if True edges with no computed traffic in the net will be ignored and not pushed in the analysis file.


def main(): 
    print "start program"
    #multi change pathes (routen müssen immer extra neu erstellt werden wegen taxiIDpickleList)
    path.simFcd=path.newPath(path.main,"simProzessiertFCD/t2MitVerschiedenenLosfahrzeiten/linkspeeds.simulatedRawFCD_-4.dat")
    path.vtypeprobe=path.newPath(path.main,"simRawFCD/t2MitVerschiedenenLosfahrzeiten/vtypeprobe_-4.out.xml")
    path.simRawFcd=path.newPath(path.main,"simRawFCD/t2MitVerschiedenenLosfahrzeiten/simulatedRawFCD_-4.out.dat")
    path.analysisWEE=path.newPath(path.main,"auswertung/reisezeit/analysisFiles/taxiAnalysisInformationWEE_-4.xml")
    arrangeData()
    print "end"
     
     
def arrangeData():
    """
    Arranges all available Data in a file which looks like:
    <vehicle id=" ">
                <step time="(s)" source=" " speed="(km/h)" edge=" " lat=" " lon=" "/> 
                ...
    </vehicle>
    """
    fcdTuple= None
    rawTuple= (None,None,None,None)
    
    #read needed files    
    print "read rawFcdDict file"; rawFcdDict=readRawFCD(path.rawFcd)
    print "read simRawFcdDict file"; simRawFcdDict=readRawFCD(path.simRawFcd, True)
    
    print "read readVtypeprobe file"; readVtypeprobe()    
    
    
    print "read fcdDict file"; fcdDict=readFCDComplete(path.fcd)
    print "read simFcdDict file"; simFcdDict=readSimFCDComplete(path.simFcd)
    
    if withoutEmptyEdges: 
        print "read drivenEdges file"; drivenEdgesSet=readRoute_EdgesMod()
        outputFile=open(path.analysisWEE,'w')
    else: outputFile=open(path.analysis,'w')
    outputFile.write("<vehicles>\n")
    
    print "write Infos"  
    #iterate over the lowest common denominator
   
    for taxiId in simFcdDict.keys():       
           outputFile.write("\t<vehicle id=\"%s\">\n" %taxiId)      
           mainId=taxiId.split("_")[0]
           #write vtypeprobe Infos           
           try:
               for vtypeTuple in vtypeDict[taxiId]:
                   #skip edges without traffic
                   if withoutEmptyEdges and vtypeTuple[1] not in drivenEdgesSet:
                       continue                    
                   outputFile.write("\t\t<step time=\"%s\" source=\"vtypeProbe\" speed=\"%s\" rawSpeed=\"%s\" edge=\"%s\" lat=\"%s\" lon=\"%s\"/>\n" 
                                    %(vtypeTuple[0],vtypeTuple[4],None,vtypeTuple[1],vtypeTuple[2],vtypeTuple[3]))
                
               #write fcd Infos (enhanced with Infos of raw-FCD)
               for fcdTuple in fcdDict[taxiId]: 
                   #skip edges without traffic
                   if withoutEmptyEdges and fcdTuple[1] not in drivenEdgesSet:
                       continue                                      
                   #search for proper tuple in raw-FCD
                   for tuple in rawFcdDict[mainId]:                   
                       if fcdTuple[0]-5<tuple[0]<fcdTuple[0]+5: #if time is equal (+/- 5 secs)
                           rawTuple=tuple
                           break  
                  
                   #write results in file
                   outputFile.write("\t\t<step time=\"%s\" source=\"FCD\" speed=\"%s\" rawSpeed=\"%s\" edge=\"%s\" lat=\"%s\" lon=\"%s\"/>\n" 
                                    %(fcdTuple[0],fcdTuple[2],rawTuple[3],fcdTuple[1],rawTuple[1],rawTuple[2]))
                   
                   #Reset values after write of line:                   
                   rawTuple= (None,None,None,None)
                
               #write simFcd Infos (enhanced with Infos of raw-FCD)                            
               for fcdTuple in simFcdDict[taxiId]:
                   #skip edges without traffic
                   if withoutEmptyEdges and fcdTuple[1] not in drivenEdgesSet:
                       continue 
                   #search for proper tuple in raw-FCD
                   for tuple in simRawFcdDict[taxiId]:                   
                       if fcdTuple[0]-2<tuple[0]<fcdTuple[0]+2: #if time is equal (+/- 5 secs)
                           rawTuple=tuple
                           break  
                   
                   #write results in file
                   outputFile.write("\t\t<step time=\"%s\" source=\"simFCD\" speed=\"%s\" rawSpeed=\"%s\" edge=\"%s\" lat=\"%s\" lon=\"%s\"/>\n" 
                                    %(fcdTuple[0],fcdTuple[2],rawTuple[3],fcdTuple[1],rawTuple[1],rawTuple[2]))
                   
                   #Reset values after write of line:                   
                   rawTuple= (None,None,None,None)
           except KeyError:
               print "Warning: taxId %s not found!" %taxiId 
           outputFile.write("\t</vehicle>\n")                 
           
     
    outputFile.write("</vehicles>\n")
    outputFile.close()


def readRawFCD(rawFcdPath, sim=False):
    """Reads the Raw-FCD-File and creates a list of Id's with a belonging List of Data tuples."""
    rawDict={}
    inputFile=open(rawFcdPath,'r')
    inputFile.seek(30)
    for line in inputFile:
        words= line.split("\t") 
        if sim: #id's of simulation raw data must be converted
            words[0]=getSimTaxiId(words[0])
              
        if words[0] in rawDict:            
            #           Veh_ID            time                     lat       lon        speed
            rawDict[words[0]].append((getTimeInSecs(words[1]), words[3], words[2], words[5][:-1]))
        else:
            #           Veh_ID            time               lat       lon        speed
            rawDict[words[0]]=[(getTimeInSecs(words[1]),words[3], words[2], words[5][:-1])]        
    inputFile.close()
    return rawDict
         
def readVtypeprobe():
    """Reads the vtypeprobe-File and creates a list of Id's with a belonging List of Data tuples."""       
    inputFile=open(path.vtypeprobe,'r')
    for line in inputFile:
        if line.find("<timestep")!=-1:
            words=line.split('"')
            time=words[1]
        elif line.find("<vehicle id=")!=-1:
            words=line.split('"')
            #set comma on 2nd position 
            words[11]=words[11][:2]+"."+words[11].replace('.','')[2:]
            words[13]=words[13][:2]+"."+words[13].replace('.','')[2:]
            if words[1] in vtypeDict:
                #        Veh_ID            time  edge             lat       lon       speed form m/s in km/h
                vtypeDict[words[1]].append((time,words[3][:-2],words[11], words[13],str(int(round(float(words[15])*3.6)))))
            else:
                #         Veh_ID      time  edge             lat       lon       speed form m/s in km/h
                vtypeDict[words[1]]=[(time,words[3][:-2],words[11], words[13],str(int(round(float(words[15])*3.6))))]
                  
                             
def testFCDUniqueness():
    "Test if for every timestamp in vls-File an unique timestamp in rawFCD exists."
    taxiId="464"
    list=set()
        
    inputFile=open(path.rawFcd,'r')
    for line in inputFile:
        words= line.split("\t")
        if words[0]==taxiId and words[4]=="90":
            list.add(words[1])
    inputFile.close()
    print len(list)
    
    inputFile=open(path.fcd,'r')
    for line in inputFile:
        words= line.split("\t")
        if words[4]==taxiId:
            if words[0]in list:                
                list.remove(words[0])    
    print len(list)   
    print list             
    inputFile.close()
    
    
    
#start the program
#profile.run('main()')
main()


#!/usr/bin/env python

import os, sys, smtplib, re
from os.path import basename, join, commonprefix
from datetime import datetime

def printStatus(makeLog, makeAllLog, textTestTmp, smtpServer, out):
    failed = ""
    build = commonprefix([basename(makeLog), basename(makeAllLog)])
    print >> out, build,
    print >> out, datetime.now().ctime()
    print >> out, "--"
    print >> out, basename(makeLog)
    warnings = 0
    errors = 0
    svnLocked = False
    for l in file(makeLog):
        if ("svn: Working copy" in l and "locked" in l) or "svn: Failed" in l:
            svnLocked = True
            failed += l
        if re.search("[Ww]arn[ui]ng[: ]", l):
            warnings += 1
        if "error " in l.lower():
            errors += 1
            failed += l
    if svnLocked:
        failed += "svn up failed\n\n"
    print >> out, warnings, "warnings"
    if errors:
        print >> out, errors, "errors"
        failed += "make failed\n\n"
    print >> out, "--"
    for root, dirs, files in os.walk(textTestTmp):
        for f in files:
            if f.startswith("batchreport"):
                b = open(join(root, f))
                l = b.readline()
                if l.startswith("FAILED") or l.startswith("succeeded") or l.startswith("killed") or l.startswith("known bugs"):
                    print >> out, f, l,
                b.close()
    print >> out, "--"
    print >> out, basename(makeAllLog)
    warnings = 0
    errors = 0
    for l in file(makeAllLog):
        if re.search("[Ww]arn[ui]ng[: ]", l):
            warnings += 1
        if "error " in l.lower():
            errors += 1
            failed += l
    print >> out, warnings, "warnings"
    if errors:
        print >> out, errors, "errors"
        failed += "make debug failed\n\n"
    print >> out, "--"
    if failed:
        fromAddr = "sumo-tests@dlr.de"
        toAddr = "sumo-tests@dlr.de"
        message = """From: "%s" <%s>
To: %s
Subject: Error occurred while building

%s""" % (build, fromAddr, toAddr, failed)
        server = smtplib.SMTP(smtpServer)
        server.sendmail(fromAddr, toAddr, message)
        server.quit()

if __name__ == "__main__":
    printStatus(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4], sys.stdout)



"""
@file    roundabout.py
@author  Daniel Krajzewicz
@author  Laura Bieker
@author  Karol Stosiek
@author  Michael Behrisch
@date    2011-11-28
@version $Id: roundabout.py 14677 2013-09-11 08:30:08Z behrisch $

This file contains a Python-representation of a single roundabout.

SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
Copyright (C) 2008-2013 DLR (http://www.dlr.de/) and contributors

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""
class Roundabout:
    def __init__(self, nodes):
        self._nodes = nodes

    def getNodes(self):
        return self._nodes



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@file    rebuildConstants.py
@author  Daniel Krajzewicz
@author  Michael Behrisch
@date    2009-07-24
@version $Id: rebuildConstants.py 14722 2013-09-18 20:37:24Z behrisch $

This script extracts definitions from <SUMO>/src/traci-server/TraCIConstants.h
 and builds an according constants definition python file "constants.py".

SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
Copyright (C) 2009-2013 DLR (http://www.dlr.de/) and contributors

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""
from __future__ import print_function
import os, sys, datetime
from optparse import OptionParser

dirname = os.path.dirname(__file__)
optParser = OptionParser()
optParser.add_option("-j", "--java", action="store_true",
                     default=False, help="generate Java output")
optParser.add_option("-o", "--output", default=os.path.join(dirname, "constants.py"),
                     help="File to save constants into", metavar="FILE")
(options, args) = optParser.parse_args()


fdo = open(options.output, "w")
if options.java:
    print("/**", file=fdo)
else:
    print('"""', file=fdo)
print("""@file    %s

This script contains TraCI constant definitions from <SUMO_HOME>/src/traci-server/TraCIConstants.h
generated by "%s" on %s.

SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
Copyright (C) 2009-2013 DLR (http://www.dlr.de/) and contributors

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.""" % (os.path.basename(options.output), os.path.basename(__file__), datetime.datetime.now()), file=fdo)

if options.java:
    print("*/\n", file=fdo)
else:
    print('"""\n', file=fdo)

fdi = open(os.path.join(dirname, "..", "..", "src", "traci-server", "TraCIConstants.h"))
started = False
for line in fdi:
    if started:
        if line.find("#endif")>=0:
            started = False
            if options.java:
                fdo.write("}")
            continue
        if line.find("#define ")>=0:
            vals = line.split(" ")
            if options.java:
                line = "    public static final int " + vals[1] + " = " + vals[2].strip() + ";\n"
            else:
                line = vals[1] + " = " + vals[2]
        if options.java:
            line = line.replace("//", "    //")
        else:
            line = line.replace("//", "#")
        fdo.write(line)
    if line.find("#define TRACICONSTANTS_H")>=0:
        started = True
        if options.java:
            fdo.write("public class TraCIConstants {")
fdi.close()
fdo.close()



#!/usr/bin/env python
"""
@file    generateTLSE3Detectors.py 
@author  Daniel Krajzewicz
@author  Karol Stosiek
@author  Michael Behrisch
@date    2007-10-25
@version $Id: generateTLSE3Detectors.py 14677 2013-09-11 08:30:08Z behrisch $

SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
Copyright (C) 2009-2013 DLR (http://www.dlr.de/) and contributors

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""

import logging
import optparse
import os
import sys
import xml.dom.minidom

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
import sumolib.net

def get_net_file_directory(net_file):
    """ Returns the directory containing the net file given. """

    dirname = os.path.split(net_file)[0]
    return dirname

def open_detector_file(destination_dir, detector_file_name):
    """ Opens a new detector file in given directory. """

    return open(os.path.join(destination_dir, detector_file_name), "w")

if __name__ == "__main__":
    # pylint: disable-msg=C0103

    logging.basicConfig(level="INFO")

    option_parser = optparse.OptionParser()
    option_parser.add_option("-n", "--net-file",
                         dest="net_file",
                         help="Network file to work with. Mandatory.",
                         type="string")
    option_parser.add_option("-l", "--detector-length",
                         dest="requested_detector_length",
                         help="Length of the detector in meters "
                              "(-1 for maximal length).",
                         type="int",
                         default=250)
    option_parser.add_option("-d", "--distance-to-TLS",
                         dest="requested_distance_to_tls",
                         help="Distance of the detector to the traffic "
                              "light in meters. Defaults to 0.1m.",
                         type="float",
                         default=.1)
    option_parser.add_option("-f", "--frequency",
                         dest="frequency",
                         help="Detector's frequency. Defaults to 60.",
                         type="int",
                         default=60)
    option_parser.add_option("-o", "--output",
                         dest="output",
                         help="The name of the file to write the detector "
                              "definitions into. Defaults to e1.add.xml.",
                         type="string",
                         default="e3.add.xml")
    option_parser.add_option("-r", "--results-file",
                         dest="results",
                         help="The name of the file the detectors write "
                              "their output into. Defaults to e1output.xml.",
                         type="string",
                         default="e3output.xml")
    option_parser.set_usage("generateTLSE3Detectors.py -n example.net.xml "
                        "-l 250 -d .1 -f 60")

    (options, args) = option_parser.parse_args()
    if not options.net_file:
        print "Missing arguments"
        option_parser.print_help()
        exit()

    logging.info("Reading net...")
    network = sumolib.net.readNet(options.net_file)

    logging.info("Generating detectors...")
    detectors_xml = xml.dom.minidom.Element("additional")
    generated_detectors = 0
    for tls in network._tlss:
        for edge in sorted(tls.getEdges(), key=sumolib.net.edge.Edge.getID):
            detector_xml = xml.dom.minidom.Element("e3Detector")
            detector_xml.setAttribute("id", "e3_" + str(tls._id) + "_" + str(edge._id))
            detector_xml.setAttribute("freq", str(options.frequency))
            detector_xml.setAttribute("file", options.results)

            input_edges = network.getDownstreamEdges(edge, options.requested_detector_length, True)
            for input_edge in input_edges:
                position = input_edge[1]
                if input_edge[3]:
                    position = .1
                for lane in input_edge[0]._lanes:
                    detector_entry_xml = xml.dom.minidom.Element("detEntry")
                    detector_entry_xml.setAttribute("lane", str(lane.getID()))
                    detector_entry_xml.setAttribute("pos", str(position))
                    detector_xml.appendChild(detector_entry_xml)

            for lane in edge._lanes:
                detector_exit_xml = xml.dom.minidom.Element("detExit")
                detector_exit_xml.setAttribute("lane", str(lane.getID()))
                detector_exit_xml.setAttribute("pos", "-.1")
                detector_xml.appendChild(detector_exit_xml)

            detectors_xml.appendChild(detector_xml)
            generated_detectors += 1

    detector_file = open_detector_file(
        get_net_file_directory(options.net_file),
        options.output)
    detector_file.write(detectors_xml.toprettyxml())
    detector_file.close()

    logging.info("%d e3 detectors generated!" % (generated_detectors))






def read(file, lanes2edges=True):
    ret = {}
    fd = open(file)
    for line in fd:
        vals = line.strip().split(":")
        if lanes2edges and vals[0]=="lane":
            vals[0] = "edge"
            vals[1] = vals[1][:vals[1].rfind("_")]
        if vals[0] not in ret:
            ret[vals[0]] = set()
        ret[vals[0]].add(vals[1])
    fd.close()
    return ret
    
def write(fdo, entries):
    for t in entries:
        writeTyped(fdo, t, entries[t])

def writeTyped(fdo, typeName, entries):
    for e in entries:
        fdo.write("%s:%s" % (typeName, entries))                          



#!/usr/bin/python
#-*- encoding: utf8 -*-
  
""" Operations and classes necessary to operate on SUMO connections. """
  
import logging
import xml.dom.minidom
  
LOGGER = logging.getLogger(__name__)

class UniformDestinationWeightCalculator():
    """Calculates weight for each destination from given lane,
       redistributing weights uniformly: first redistributes the total
       traffic among incoming lanes uniformly and then divides the total
       traffic for a lane among all of the destination lanes.
       
       For example, assume we  have a junction with one incoming road In1
       with 3 incoming lanes and 2 outgoing roads Out1 and Out2,
       having 2 and 3 outgoing lanes, respectively.
  
                   ___________________
            lane 0 _______     _______ lane 0
       Out2 lane 1 _______     _______ lane 1  In1
            lane 2 _______     _______ lane 2
                          | | |
                          | | |
                          | | |
                      lane 0 1
                           Out1
  
      Assume that lanes are connected as follows:
      - In1, lane 0 is connected to Out2, lane 0
      - In1, lane 1 is connected to Out2, lane 1
      - In1, lane 2 is connected to Out2, lane 2
      
      - In1, lane 1 is connected to Out1, lane 0
      - In1, lane 2 is connected to Out1, lane 1
  
       This weight calculator will redistribute weights as follows:
  
       1. Distribute the incoming traffic on In1 uniformly on lanes 0, 1 and 2:
          as a result, each lane has been assigned 33,(3)% of the traffic.
       2. Since In1, lane 0 is connected to Out2, lane 0, whole traffic
          from In1, lane 0 is redirected to Out2, lane 0, which makes 33,(3)%
          of the total incoming traffic from In1.
       3. Since In1, lane 1 is connected both to Out2, lane 1 and Out1, lane 0,
          the traffic on that lane (that is, 33,(3)% of the total traffic) is
          spread uniformly among these two lanes, resulting in 16,(6)% of the total
          traffic for each destination lane.
       4. Similarly, In2, lane 2's traffic is spread uniformly among Out2, lane 2
          and Out 1, lane 1, resulting in 16,(6)% of the total traffic for each
          destination lane.
  
       This, if we ask the calculator for weight assigned to In1, lane 0, we get
       33,(3) as a result; if we ask for weight assigned to In1, lane 1, we get
       16,(6) as a result.
  
       Note: If you want to provide your own weight calculator (based on Gaussian
       distribution, for example), simply provide a class with calculate_weight
       method. The method's signature may need to be changed in that case. """
  
    # pylint: disable=R0903

    logger = logging.getLogger(__name__)

    def __init__(self):
        pass
  
    def calculate_weight(self,
                         source_lane_no,
                         source_total_lanes,
                         destination_total_lanes):
  
        """ Calculates the weight assigned to a single destination 
            from given lane in an uniform way. See class docs
            for explanation. """
  
        lane_weight = 100.0 / source_total_lanes
        destination_weight = lane_weight / destination_total_lanes
  
        self.logger.debug("Destination weight for lane %s/%s connected to %s "
            "destinations: %f" %
                (str(source_lane_no), str(source_total_lanes),
                    str(destination_total_lanes), destination_weight))
  
        return destination_weight
  
class Connections:
    """ Represents all of the connections in the network we have read. """
  
    logger = logging.getLogger(__name__)
  
    def __init__(self,
                 calculator = UniformDestinationWeightCalculator()):
        """ Constructor. Allows providing own destination weight calculator,
            which defaults to UniformDestinationWeightCalculator if not "
            provided. """
  
        self.connections_map = { }
        self.destination_weight_calculator = calculator
  
    def add(self,
            source,
            source_lane,
            destination):
  
        """ Adds a connection. If a connection is readded, 
            a warning is issued. """
  
        self.logger.debug("Adding connection %s (%s) -> %s" %
            (str(source), str(source_lane), str(destination)))
  
        if source not in self.connections_map:
            self.logger.debug("Created new mapping for %s" %
                (str(source)))
            self.connections_map[source] = {}
  
        if source_lane not in self.connections_map[source]:
            self.logger.debug("Created new mapping for %s / %s" %
                (str(source), str(source_lane)))
            self.connections_map[source][source_lane] = set()
  
        if destination in self.connections_map[source][source_lane]:
            self.logger.warn("Destination for %s (lane %s) readded: %s"
                % (str(source), str(source_lane), str(destination)))
  
        self.connections_map[source][source_lane].add(destination)
  
    def get_sources(self):
        """ Returns all of the incoming edges that this connections collection
            contains. Incoming edges are sorted alphabetically
            in ascending order. """
  
        sources = self.connections_map.keys()
        sources.sort()
        return sources
  
    def get_lanes(self, source):
        """ Returns all lanes that have connections for the given edge.
            The connection_source must have been added before. """
  
        return self.connections_map[source].keys()
  
    def get_destinations(self, source, source_lane):
        """ Returns all possible destinations that are achievable from given
            lane on given edge. The connection_source
            and connection_source_lane must have been added before. """
  
        return self.connections_map[source][source_lane]
  
    def calculate_destination_weight(self,
                                     source,
                                     source_lane,
                                     destination):
        """ Calculates weight assigned to the given destination using
            weight calculator provided in class' constructor.
            The connection_source, connection_source_lane
            and connection_destination must have been added before."""
  
        weight = self.destination_weight_calculator.calculate_weight(
            source_lane,
            len(self.get_lanes(source)),
            len(self.get_destinations(source, source_lane)))
  
        self.logger.debug("Destination weight for connection "
            "%s (%s) -> %s: %f" %
                (str(source), str(source_lane), str(destination), weight))
  
        return weight
  
class ConnectionXML():
    """ Represents a raw, xml-defined connection. """
  
    def __init__(self, connection_xml):
        """ Constructor. The connection_xml argument must be a well-formed
            connection XML entity string. See SUMO docs for information on
            connection XML format. """
  
        self.connection_xml = connection_xml
        
    def get_source(self):
        """ Returns the connection's incoming edge. Raises AttributeError
            if the incoming edge is missing. """
  
        source = self.connection_xml.getAttribute("from")
        if source is '':
            raise AttributeError
        return source
  
    def get_source_lane(self):
        """ Returns the connection's incoming edge's lane.
            Raises AttributeError if the incoming edge's lane is missing. """
  
        source_lane = self.connection_xml.getAttribute("fromLane")
        if source_lane is '':
            raise AttributeError
        return source_lane
  
    def get_destination(self):
        """ Returns the connection's outgoing edge. Raises AttributeError
            if the outgoing edge is missing. """
  
        destination = self.connection_xml.getAttribute("to")
        if destination is '':
            raise AttributeError
        return destination
      
  
class ConnectionsXMLReader():
    """ Reads the XML connection definitions. """
    # pylint: disable=R0903

    logger = logging.getLogger("ConnectionsXMLReader")
  
    def __init__(self, connections_xml_stream):
        """ Constructor. The connections_xml_stream argument may be
            a filename or an opened file. """
  
        self.connections_xml = xml.dom.minidom.parse(connections_xml_stream)
  
    def get_connections(self):
        """ Returns connections defined in the XML stream provided in
            the constructor. """
  
        return [ ConnectionXML(connection_xml) for connection_xml in
            self.connections_xml.getElementsByTagName("connection") ]
  
  
  
def from_stream(input_connections):
    """ Constructs Connections object from connections defined in the given
        stream. The input_connections argument may be a filename or an opened
        file. """
  
    LOGGER.info("Reading connections from input stream")

    connections = Connections()
    connections_xml_reader = ConnectionsXMLReader(input_connections)
    for xml_connection in connections_xml_reader.get_connections():
        connections.add(xml_connection.get_source(),
                        xml_connection.get_source_lane(),
                        xml_connection.get_destination())
   
    return connections
  
import collectinghandler
import unittest
  
  
class UniformDestinationWeightCalculatorTestCase(unittest.TestCase):
    # pylint: disable=C,W,R,E,F 

    def setUp(self):
      self.calculator = UniformDestinationWeightCalculator()
  
    def tearDown(self):
      self.calculator = None
  
    def test_uniform_weight_spread_across_lanes(self):
      # 1 lane case 
      self.assertAlmostEqual(100.0, self.calculator.calculate_weight(0, 1, 1))
  
      # 2 lane case 
      self.assertAlmostEqual(50.0, self.calculator.calculate_weight(0, 2, 1))
      self.assertAlmostEqual(50.0, self.calculator.calculate_weight(1, 2, 1))
  
      # 2 lane case 
      self.assertAlmostEqual(100.0/3, self.calculator.calculate_weight(0, 3, 1))
      self.assertAlmostEqual(100.0/3, self.calculator.calculate_weight(1, 3, 1))
      self.assertAlmostEqual(100.0/3, self.calculator.calculate_weight(2, 3, 1))
  
    def test_uniform_weight_spread_across_destinations(self):
      # 1 destination
      self.assertAlmostEqual(100.0, self.calculator.calculate_weight(0, 1, 1))
  
      # 2 destinations
      self.assertAlmostEqual(100.0/2, self.calculator.calculate_weight(0, 1, 2))
  
      # 3 destinations
      self.assertAlmostEqual(100.0/3, self.calculator.calculate_weight(0, 1, 3))
  
  
class ConnectionsTestCase(unittest.TestCase):
    # pylint: disable=C,W,R,E,F 

    def setUp(self):
      self.connections = Connections()
  
    def tearDown(self):
      self.connections = None
  
    def assert_contains_source(self, source):
      self.assertTrue(source in
          self.connections.get_sources())
  
    def assert_contains_lane(self, source, lane):
      self.assertTrue(lane in
          self.connections.get_lanes(source))
  
    def assert_contains_destination(self, source, lane, destination):
      self.assertTrue(destination in
          self.connections.get_destinations(source, lane))
    
    def assert_sources_no(self, sources_no):
      self.assertEqual(sources_no,
                       len(self.connections.get_sources()))
  
    def assert_lanes_no(self, source, lanes_no):
      self.assertEqual(lanes_no,
                       len(self.connections.get_lanes(source)))
  
    def assert_destinations_no(self, source, lane, destinations_no):
      self.assertEqual(destinations_no,
                       len(self.connections.get_destinations(source, lane)))
    
    def test_map_empty_on_init(self):
      self.assert_sources_no(0)
  
    def test_add(self):
      self.connections.add("source", "source lane", "destination")
      
      self.assert_sources_no(1)
      self.assert_contains_source("source")
      
      self.assert_lanes_no("source", 1)
      self.assert_contains_lane("source", "source lane")
  
      self.assert_destinations_no("source", "source lane", 1)
      self.assert_contains_destination("source", "source lane", "destination") 
  
    def test_add_different_lanes(self):
      self.connections.add("source", "source lane 1", "destination")
      self.connections.add("source", "source lane 2", "destination")
  
      self.assert_sources_no(1)
      self.assert_contains_source("source")
      
      self.assert_lanes_no("source", 2)
      self.assert_contains_lane("source", "source lane 1")
      self.assert_contains_lane("source", "source lane 2")
  
      self.assert_destinations_no("source", "source lane 1", 1)
      self.assert_contains_destination("source", "source lane 1", "destination")
  
      self.assert_destinations_no("source", "source lane 2", 1)
      self.assert_contains_destination("source", "source lane 2", "destination")
   
    def test_add_different_destinations(self):
      self.connections.add("source", "source lane", "destination 1")
      self.connections.add("source", "source lane", "destination 2")
  
      self.assert_sources_no(1)
      self.assert_contains_source("source")
      
      self.assert_lanes_no("source", 1)
      self.assert_contains_lane("source", "source lane")
  
      self.assert_destinations_no("source", "source lane", 2)
      self.assert_contains_destination("source", "source lane", "destination 1")
      self.assert_contains_destination("source", "source lane", "destination 2")
   
    def test_readd_raises_warning(self):
      collecting_handler = collectinghandler.CollectingHandler()
      self.connections.logger.addHandler(collecting_handler)
  
      self.connections.add("source", "source lane", "destination")    
      self.connections.add("source", "source lane", "destination")
      
      self.assert_sources_no(1)
      self.assert_contains_source("source")
      
      self.assert_lanes_no("source", 1)
      self.assert_contains_lane("source", "source lane")
  
      self.assert_destinations_no("source", "source lane", 1)
      self.assert_contains_destination("source", "source lane", "destination")
  
      self.assertTrue(
          "Destination for source (lane source lane) readded: destination"
              in [record.getMessage() for record in collecting_handler.log_records])
  
  
class ConnectionXMLTestCase(unittest.TestCase):
    # pylint: disable=C,W,R,E,F 

    SOURCE_ATTRIBUTE = "from"
    SOURCE_LANE_ATTRIBUTE= "fromLane"
    DESTINATION_ATTRIBUTE = "to"
    DESTINATION_LANE_ATTRIBUTE = "toLane"
   
    def setUp(self):
      # Shared resources.
      self.from_value = "source"
      self.to_value = "destination"
      self.from_lane_value = "source lane"
      self.to_lane_value = "destination lane"
      
    def set_up_new_connection_element(self):
      connection_element = xml.dom.minidom.Element("connection")
      connection_element.setAttribute(self.SOURCE_ATTRIBUTE,
          self.from_value)
      connection_element.setAttribute(self.SOURCE_LANE_ATTRIBUTE,
          self.from_lane_value)
      connection_element.setAttribute(self.DESTINATION_ATTRIBUTE,
          self.to_value)
      connection_element.setAttribute(self.DESTINATION_LANE_ATTRIBUTE,
          self.to_lane_value)
      return connection_element
  
    def set_up_connection_xml_without(self, attribute):
      connection_element = self.set_up_new_connection_element()
      connection_element.removeAttribute(attribute)
      self.set_up_connection_xml(connection_element)
  
    def set_up_connection_xml(self, connection_element):
      self.connection_xml = ConnectionXML(connection_element)
  
    def tearDown(self):
      self.connection_xml = None
  
    def test_get_source(self):
      self.set_up_connection_xml(self.set_up_new_connection_element())
      self.assertEqual(self.from_value, self.connection_xml.get_source())
  
    def test_get_source_with_missing_source(self):
      self.set_up_connection_xml_without(self.SOURCE_ATTRIBUTE)
      self.assertRaises(AttributeError, self.connection_xml.get_source)
  
    def test_get_source_lane(self):
      self.set_up_connection_xml(self.set_up_new_connection_element())
      self.assertEqual(self.from_lane_value, self.connection_xml.get_source_lane())
  
    def test_get_source_lane_with_missing_source_lane(self):
      self.set_up_connection_xml_without(self.SOURCE_LANE_ATTRIBUTE)
      self.assertRaises(AttributeError, self.connection_xml.get_source_lane)
  
    def test_get_destination(self):
      self.set_up_connection_xml(self.set_up_new_connection_element())
      self.assertEqual(self.to_value, self.connection_xml.get_destination())
  
    def test_get_destination_with_missing_destination(self):
      self.set_up_connection_xml_without(self.DESTINATION_ATTRIBUTE)
      self.assertRaises(AttributeError, self.connection_xml.get_destination)
  
if __name__ == "__main__":
    unittest.main()
  



                    
import os, subprocess, sys, random, helpers
from matplotlib import rcParams
from pylab import *
from matplotlib.ticker import FuncFormatter as ff
import numpy as np

sys.path.append(os.path.join(os.path.dirname(__file__), '..', '..', 'tools'))
sys.path.append(os.path.join(os.environ.get("SUMO_HOME", os.path.join(os.path.dirname(__file__), '..', '..')), 'tools'))
import sumolib.output





def main(args=None):
  """The main function; parses options and plots"""
  ## ---------- build and read options ----------
  from optparse import OptionParser
  optParser = OptionParser()
  optParser.add_option("-i", "--tripinfos-inputs", dest="tripinfos", metavar="FILE",
                         help="Defines the tripinfo-output files to use as input")
  optParser.add_option("-v", "--verbose", dest="verbose", action="store_true",
                         default=False, help="If set, the script says what it's doing")
  optParser.add_option("-m", "--measure", dest="measure", 
                         default="duration", help="Define which measure to plot")
  optParser.add_option("--bins", dest="bins", 
                         type="int", default=20, help="Define the bin number")
  optParser.add_option("--minV", dest="minV", 
                         type="float", default=None, help="Define the minimum boundary")
  optParser.add_option("--maxV", dest="maxV", 
                         type="float", default=None, help="Define the maximum boundary")
  # standard plot options
  helpers.addInteractionOptions(optParser)
  helpers.addPlotOptions(optParser)
  # parse
  options, remaining_args = optParser.parse_args(args=args)

  if options.tripinfos==None:
    print "Error: at least one tripinfo file must be given"
    sys.exit(1)

  minV = None
  maxV = None
  files = options.tripinfos.split(",")
  values = {}
  for f in files:
    if options.verbose: print "Reading '%s'..." % f
    nums = sumolib.output.parse_sax__asList(f, "tripinfo", [options.measure])
    fv = sumolib.output.toList(nums, options.measure)
    sumolib.output.prune(fv, options.minV, options.maxV) 
    values[f] = fv 
    if minV==None: 
      minV = fv[0]
      maxV = fv[0]
    minV = min(minV, min(fv))
    maxV = max(maxV, max(fv))
  
  hists = {}
  binWidth = (maxV-minV+1) / float(options.bins)
  for f in files:
    h = [0] * options.bins
    for v in values[f]:
      i = int((v-minV)/binWidth)
      h[i] = h[i] + 1
    hists[f] = h
  
  width = binWidth / float(len(files)) * .8
  offset = binWidth * .1
  center = []
  for j in range(0, options.bins):
    center.append(binWidth*j+offset)

  fig, ax = helpers.openFigure(options)
  for i,f in enumerate(files):
    c = helpers.getColor(options, i, len(files))
    l = helpers.getLabel(f, i, options)
    bar(center, hists[f], width = width, label=l, color=c)
    for j in range(0, options.bins):
      center[j] = center[j] + width
  helpers.closeFigure(fig, ax, options)

if __name__ == "__main__":
  sys.exit(main(sys.argv))
    


# -*- coding: Latin-1 -*-
"""
@file    TaxisPerEdge.py
@author  Sascha Krieg
@author  Daniel Krajzewicz
@author  Michael Behrisch
@date    2008-04-08

Counts for every edge in the given FCD-file the number of Taxis which used this edge.
After that this information can be visualized with an script called mpl_dump_onNet from Daniel.


SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
Copyright (C) 2008-2013 DLR (http://www.dlr.de/) and contributors

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""

import util.Path as path

#global vars
edgeList={}



def main():    
    print "start program"     
    countTaxisForEachEdge()
    writeOutput()
    print "end"

    
def countTaxisForEachEdge():
    """counts the frequency of each edge"""
    inputFile=open(path.vls,'r')
    for line in inputFile:
        words= line.split("\t")
        edgeList.setdefault(words[1],set())
        edgeList[words[1]].add(words[4])
    
    for k in edgeList:     
        print k
        print len(edgeList[k])
    print len(edgeList)


def writeOutput():
    """Writes an XML-File with the extracted results"""
    outputFile=open(path.taxisPerEdge,'w')
    outputFile.write("<netstats>\n")
    outputFile.write("\t<interval begin=\"0\" end=\"899\" id=\"dump_900\">\n")
    for k in edgeList:  
        outputFile.write("\t\t<edge id=\"%s\" no=\"%s\" color=\"1.0\"/>\n" %(k, len(edgeList[k])))        
    outputFile.write("\t</interval>\n")        
    outputFile.write("</netstats>")

    
#start the program
main()


#!/usr/bin/env python
"""
@file    runner.py
@author  Michael Behrisch
@author  Daniel Krajzewicz
@date    2011-10-28
@version $Id: runner.py 14678 2013-09-11 08:53:06Z behrisch $

This script is a test runner for the CityMobil scenario.

SUMO, Simulation of Urban MObility; see http://sumo.sourceforge.net/
Copyright (C) 2011-2012 DLR (http://www.dlr.de/) and contributors

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""


import os, sys
import shutil
import vehicleControl, simpleManager, agentManager

# build/check network
import createNet
# perform simple scenario
vehicleControl.init(simpleManager.SimpleManager(), True)
# perform agent scenario
vehicleControl.init(agentManager.AgentManager(), True)
try: shutil.copy("all-the-results.txt", "../result2")
except: print "Missing 'all-the-results.txt'"



#!/usr/bin/python
"""
@file    0123to0130.py
@author  Daniel Krajzewicz
@author  Jakob Erdmann
@author  Michael Behrisch
@date    2007
@version $Id: 0123to0130.py 14677 2013-09-11 08:30:08Z behrisch $

Changes xml network files from version 0.12.3 to version 0.13.0.

SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
Copyright (C) 2009-2013 DLR (http://www.dlr.de/) and contributors

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""
import os, string, sys, glob
import xml
from xml.sax import parse, handler
from optparse import OptionParser
from collections import defaultdict

# attributes sorting lists
a = {}
a['net'] = ('version', )
a['edge'] = ( 'id', 'from', 'to', 'name', 'priority', 'type', 'function', 'spread_type', 'shape' )
a['lane'] = ( 'id', 'index', 'vclasses', 'allow', 'disallow', 'maxspeed', 'maxSpeed', 'length', 'endOffset', 'width', 'shape' )
a['junction'] = ( 'id', 'type', 'x', 'y', 'incLanes', 'intLanes', 'shape' )
a['logicitem'] = ('response', 'foes', 'cont' )
a['succlane'] = ('via', 'tl', 'linkno', 'dir', 'state' )
a['connection'] = ('from', 'to', 'fromLane', 'toLane', 'via', 'tl', 'linkIdx', 'dir', 'state' )
a['row-logic'] = a['ROWLogic'] = ( 'id', 'requestSize' )
a['tl-logic'] = a['tlLogic'] = ( 'id', 'type', 'programID', 'offset' )
a['location'] = ( 'netOffset', 'convBoundary', 'origBoundary', 'projParameter' )
a['phase'] = ( 'duration', 'state', 'min_dur', 'max_dur' )
a['district'] = ( 'id', 'shape', 'edges' )
a['dsink'] = ( 'id', 'weight' )
a['dsource'] = ( 'id', 'weight' )
a['taz'] = ( 'id', 'shape', 'edges' )
a['tazSink'] = ( 'id', 'weight' )
a['tazSource'] = ( 'id', 'weight' )
a['roundabout'] = ( 'nodes', 'dummy' )
a['request'] = ( 'index', 'response', 'foes', 'cont' )
a['succ'] = ( 'edge', 'lane', 'junction')

# attributes which are optional
b = defaultdict(dict)
b['edge']['type'] = ''
b['edge']['function'] = 'normal'
b['succlane']['tl'] = ''
b['succlane']['linkno'] = ''
b['succlane']['linkIndex'] = ''
b['junction']['shape'] = ''

# elements which are single (not using opening/closing tag)
SINGLE = ( 'roundabout', 'logicitem', 'phase', 'succlane', 'dsource', 'dsink', 'location', 
           'lane', 'timed_event', 'connection', 'request' )

# remove these
REMOVED = ( 'lanes', 'logic', 'succ', 'row-logic', 'ROWLogic', 'logicitem')

# renamed elements
RENAMED_TAGS = {'tl-logic': 'tlLogic', 
        'row-logic': 'ROWLogic', 
        'district':'taz', 
        'dsource':'tazSource', 
        'dsink':'tazSink', 
        'succlane':'connection'}

renamedAttrs = {'min_dur': 'minDur', 
        'max_dur': 'maxDur', 
        'spread_type': 'spreadType', 
        'maxspeed':'speed', 
        'maxSpeed':'speed', 
        'linkIdx':'linkIndex', 
        'linkno':'linkIndex'}

renamedValues = {'state': {'t': 'o'}, 'type':{'DEAD_END': 'dead_end'} }

def getBegin(file):
    fd = open(file)
    content = "";
    for line in fd:
        if '<net' in line or '<districts' in line or '<add' in line:
            fd.close()
            return content
        if '<?xml version' in line:
            continue
        content = content + line
    fd.close()
    return ""

class NetConverter(handler.ContentHandler):
    def __init__(self, outFileName, begin, has_no_destination):
        self._out = open(outFileName, "w")
        self._out.write('<?xml version="1.0" encoding="iso-8859-1"?>\n')
        self._out.write(begin)
        self._tree = []
        self._content = ""
        self._am_parsing_rowlogic = None # parsing old element
        self._logicitems = defaultdict(list) # maps junction ids to items
        self._laneCount = 0
        self._has_no_destination = has_no_destination

    def isSingle(self, name):
        if name in SINGLE:
            return True
        if name == "junction" and self._single_junction:
            return True
        return False

    def remove(self, name):
        if name == "succ" and self._has_no_destination:
            return False
        if name in REMOVED:
            return True
        else:
            return False

    def rename(self, name):
        if name == "succlane" and self._has_no_destination:
            return False
        if name in RENAMED_TAGS:
            return True
        else:
            return False

    def checkWrite(self, what):
        self._out.write(what.encode('iso-8859-1'))

    def indent(self):
        self._out.write(" " * (4*len(self._tree)))

    def endDocument(self):
        self.checkWrite("\n")
        self._out.close()

    def startElement(self, name, attrs):
        # special preparations
        if name == "succ":
            self._succ_from = attrs["edge"]
            self._succ_fromIdx = attrs["lane"].split('_')[-1]
        if name == "row-logic" or name == "ROWLogic":
            self._am_parsing_rowlogic = attrs["id"]
        if name == "logicitem": 
            self._logicitems[self._am_parsing_rowlogic].append(attrs)
        if name == "edge":
            self._laneCount = 0
        if name == "junction":
            self._single_junction = (attrs["id"][0] == ":" 
                    or attrs["type"] == 'DEAD_END'
                    or attrs["type"] == 'dead_end')
        # skip removed
        if self.remove(name):
            return
        # compress empty lines
        if self._content.find("\n\n")>=0:
            self.checkWrite("\n")
        self._content = ""
        # open tag
        self.indent()
        if self.rename(name):
            self.checkWrite("<" + RENAMED_TAGS[name])
        else:
            self.checkWrite("<" + name)
        # special attribute handling
        if name == "succlane":
            if self._has_no_destination:
                # keep original
                self.checkWrite(' lane="%s"' % attrs["lane"])
            else:
                # upgrade
                self.checkWrite(' from="%s"' % self._succ_from)
                sepIndex = attrs['lane'].rindex('_')
                toEdge = attrs['lane'][:sepIndex]
                toIdx = attrs['lane'][sepIndex+1:]
                self.checkWrite(' to="%s"' % toEdge)
                self.checkWrite(' fromLane="%s"' % self._succ_fromIdx)
                self.checkWrite(' toLane="%s"' % toIdx)
        # write attributes
        if name == "net":
            self.checkWrite(' version="0.13" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="http://sumo.sf.net/xsd/net_file.xsd"')
        else:
            if name in a:
                for key in a[name]:
                    val = None
                    if key in renamedAttrs and attrs.has_key(renamedAttrs[key]):
                        key = renamedAttrs[key]
                    if attrs.has_key(key):
                        val = attrs[key]
                        if key in renamedValues:
                            val = renamedValues[key].get(val, val)
                        if name == "succlane" and (key == "linkno" or key == "linkIndex") and attrs["tl"] == '':
                            val = ''
                        if name not in b or key not in b[name] or val!=b[name][key]:
                            self.checkWrite(' ' + renamedAttrs.get(key, key) + '="%s"' % val)
                    elif name == "lane" and key == "index":
                        self.checkWrite(' %s="%s"' % (key,self._laneCount))
                        self._laneCount += 1
                    elif name == "connection" and key == "fromLane":
                        self.checkWrite(' fromLane="%s" toLane="%s"' % tuple(attrs["lane"].split(":")))
        # close tag
        if self.isSingle(name):
            self.checkWrite("/>\n")
        else:
            self.checkWrite(">\n")
        self._tree.append(name)
        # transfer items from removed element ROWLogic
        if name == "junction":
            for logicitem_attrs in self._logicitems[attrs["id"]]:
                self.indent()
                self.checkWrite("<request")
                self.checkWrite(' index="%s"' % logicitem_attrs["request"])
                for key in a["logicitem"]:
                    if logicitem_attrs.has_key(key): # cont is optional
                        self.checkWrite(' ' + key + '="%s"' % logicitem_attrs[key])
                self.checkWrite("/>\n")

    def endElement(self, name):
        # special preparations
        if name == "row-logic" or name == "ROWLogic":
            self._am_parsing_rowlogic = None
        # skip removed
        if self.remove(name):
            return
        # optionaly write closing tag
        self._tree.pop()
        if name=="net":
            self.checkWrite("\n")
        if not self.isSingle(name):
            self.indent()
            if self.rename(name):
                self.checkWrite("</" + RENAMED_TAGS[name] + ">")
            else:
                self.checkWrite("</" + name + ">")
            if name!="net":
                self.checkWrite("\n")

    def characters(self, content):
        self._content += content

    def processingInstruction(self, target, data):
        self.checkWrite('<?%s %s?>' % (target, data))

def changeFile(fname):
    if options.verbose:
        print "Patching " + fname + " ..."
    if (("_deprecated_" in fname and not "net.netconvert" in fname) or 
            (os.path.join('tools','net','0') in fname)):
        print "Skipping file (looks like intentionally deprecated input): " + fname
        return
    has_no_destination = False
    if "SUMO_NO_DESTINATION" in open(fname).read():
        print "Partial conversion (cannot convert SUMO_NO_DESTINATION): " + fname
        has_no_destination = True
    net = NetConverter(fname+".chg", getBegin(fname), has_no_destination)
    try:
        parse(fname, net)
    except xml.sax._exceptions.SAXParseException:
        print "Could not parse '%s' maybe it contains non-ascii chars?" % fname
    if options.inplace:
        os.remove(fname)
        os.rename(fname+".chg", fname)

def changeEdgeFile(fname):
    if options.verbose:
        print "Patching " + fname + " ..."
    out = open(fname+".chg", 'w')
    for line in open(fname):
        if "<lane" in line:
            line = line.replace(" id", " index")
        if "<edge" in line:
            line = line.replace(" fromnode", " from").replace(" tonode", " to")
            line = line.replace(" spread_type", " spreadType").replace(" maxspeed", " maxSpeed")
            line = line.replace(" nolanes", " numLanes")
        out.write(line)
    out.close()
    if options.inplace:
        os.remove(fname)
        os.rename(fname+".chg", fname)

def changeConnectionFile(fname):
    if options.verbose:
        print "Patching " + fname + " ..."
    out = open(fname+".chg", 'w')
    for line in open(fname):
        if "<connection" in line and "lane" in line:
            line = line.replace(" lane", " fromLane")
        for i in range(10):
            line = line.replace(':%s"' % i, '" toLane="%s"' % i)
        out.write(line)
    out.close()
    if options.inplace:
        os.remove(fname)
        os.rename(fname+".chg", fname)

def walkDir(srcRoot):
    for root, dirs, files in os.walk(srcRoot):
        for name in files:
            if name.endswith(".net.xml") or name in ["net.netconvert", "net.netgen",
                                                       "tls.scenario", "net.scenario"]:
                changeFile(os.path.join(root, name))
            if options.edges and name.endswith(".edg.xml"):
                changeEdgeFile(os.path.join(root, name))
            elif options.connections and name.endswith(".con.xml"):
                changeConnectionFile(os.path.join(root, name))
        for ignoreDir in ['.svn', 'foreign']:
            if ignoreDir in dirs:
                dirs.remove(ignoreDir)


optParser = OptionParser()
optParser.add_option("-v", "--verbose", action="store_true",
                     default=False, help="tell me what you are doing")
optParser.add_option("-i", "--inplace", action="store_true",
                     default=False, help="replace original files")
optParser.add_option("-e", "--edges", action="store_true",
                     default=False, help="include edge XML files")
optParser.add_option("-c", "--connections", action="store_true",
                     default=False, help="include connection XML files")
(options, args) = optParser.parse_args()

if len(args) == 0:
    print "Usage: " + sys.argv[0] + " <net>+"
    sys.exit()
for arg in args:
    for fname in glob.glob(arg):
        if os.path.isdir(fname):
            walkDir(fname)
        else:
            if options.edges and fname.endswith(".edg.xml"):
                changeEdgeFile(fname)
            elif options.connections and fname.endswith(".con.xml"):
                changeConnectionFile(fname)
            else:
                changeFile(fname)



#!/usr/bin/env python
"""
@file    netdumpmean.py
@author  Daniel Krajzewicz
@author  Michael Behrisch
@date    2007-10-25
@version $Id: netdumpmean.py 14677 2013-09-11 08:30:08Z behrisch $


This script reads two network dumps,
 computes the mean values
 and writes the results into the output file

SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
Copyright (C) 2008-2013 DLR (http://www.dlr.de/) and contributors

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""
import os, string, sys
from optparse import OptionParser
from xml.sax import saxutils, make_parser, handler
from datetime import datetime


class WeightsReader(handler.ContentHandler):
    """Reads the dump file"""

    def __init__(self):
        self._id = ''
        self._edgeValues = {}
        self._intervalBegins = []
        self._intervalEnds = []

    def startElement(self, name, attrs):
        if name == 'interval':
            self._beginTime = int(attrs['begin'])
            self._intervalBegins.append(self._beginTime)
            self._intervalEnds.append(int(attrs['end']))
            self._edgeValues[self._beginTime] = {}
        if name == 'edge':
            self._id = attrs['id']
            self._edgeValues[self._beginTime][self._id] = {}
            for attr in attrs.getQNames():
                if attr!="id":
                    self._edgeValues[self._beginTime][self._id][attr] = float(attrs[attr])
    
    def sub(self, weights, exclude):
         for t in self._edgeValues:
             for e in self._edgeValues[t]:
                 for a in self._edgeValues[t][e]:
                     if a not in exclude:
                         self._edgeValues[t][e][a] = (self._edgeValues[t][e][a] + weights._edgeValues[t][e][a]) / 2.
 
    def write(self, options):
        fd = open(options.output, "w")
        fd.write("<?xml version=\"1.0\"?>\n\n")
        fd.write("<!-- generated on %s by netdumpdiv.py \n" % datetime.now())
        fd.write("   -1 %s\n" % options.dump1)
        fd.write("   -2 %s\n" % options.dump2)
        fd.write("   -o %s\n" % options.output)
        fd.write("-->\n\n")
        fd.write("<netstats>\n")
        for i in range(0, len(self._intervalBegins)):
            fd.write("   <interval begin=\"%s\" end=\"%s\">\n" % (self._intervalBegins[i], self._intervalEnds[i]))
            t = self._intervalBegins[i]
            for e in self._edgeValues[t]:
                fd.write("      <edge id=\"%s\"" % e)
                for a in self._edgeValues[t][e]:
                    fd.write(" %s=\"%s\"" % (a, self._edgeValues[t][e][a]))
                fd.write("/>\n")
            fd.write("   </interval>\n")
        fd.write("</netstats>\n")


# initialise 
optParser = OptionParser()
optParser.add_option("-v", "--verbose", action="store_true", dest="verbose",
                     default=False, help="tell me what you are doing")
    # i/o
optParser.add_option("-1", "--dump1", dest="dump1",
                     help="First dump (mandatory)", metavar="FILE")
optParser.add_option("-2", "--dump2", dest="dump2",
                     help="Second  dump (mandatory)", metavar="FILE")
optParser.add_option("-o", "--output", dest="output",
                     help="Name for the output", metavar="FILE")
optParser.add_option("-e", "--exclude", dest="exclude",
                     help="Exclude these values from being changed (stay as in 1)", metavar="FILE")
# parse options
(options, args) = optParser.parse_args()


parser = make_parser()
# read dump1
if options.verbose:
    print "Reading dump1..."
weights1 = WeightsReader()
parser.setContentHandler(weights1)
parser.parse(options.dump1)
# read dump2
if options.verbose:
    print "Reading dump2..."
weights2 = WeightsReader()
parser.setContentHandler(weights2)
parser.parse(options.dump2)
# process
if options.verbose:
    print "Computing mean..."
exclude = []
if options.exclude:
    exclude = options.exclude.split(",")
weights1.mean(weights2, exclude)
# save
if options.verbose:
    print "Writing..."
weights1.write(options)




"""
@file    phem.py
@author  Daniel Krajzewicz
@date    2013-01-15
@version $Id: phem.py 14732 2013-09-20 20:11:11Z behrisch $

This module includes functions for converting SUMO's fcd-output into
data files read by PHEM.

SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
Copyright (C) 2013 DLR (http://www.dlr.de/) and contributors

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""
from __future__ import print_function
import math
import sumolib.output
import sumolib.net

def _convType(tID):
  if tID.lower().startswith("passenger") or tID.lower().startswith("PKW"):
    return "PKW"
  if tID.lower().startswith("bus"):
    return "BUS"
  if tID.lower().startswith("heavy") or tID.lower().startswith("LKW"):
    return "LKW"
  print("Could not convert the vehicle type properly")
  return "unknown"

        
def fcd2dri(inpFCD, outSTRM, ignored):
  """
  Reformats the contents of the given fcd-output file into a .dri file, readable
  by PHEM. The fcd-output "fcd" must be a valid file name of an fcd-output.

  The following may be a matter of changes:
  - the engine torque is not given 
  """
  #print >> outSTRM, "v1\n<t>,<v>,<grad>,<n>\n[s],[km/h],[%],[1/min]\n"
  print("v1\n<t>,<v>,<grad>\n[s],[km/h],[%]", file=outSTRM)
  for q in inpFCD:
    if q.vehicle:
      for v in q.vehicle:
        percSlope = math.sin(float(v.slope))*100.
        print("%s,%s,%s" % (sumolib._intTime(q.time), float(v.speed)*3.6, percSlope), file=outSTRM)


def net2str(net, outSTRM):
  """
  Writes the network object given as "inpNET" as a .str file readable by PHEM.
  Returns a map from the SUMO-road id to the generated numerical id used by PHEM.

  The following may be a matter of changes:
  - currently, only the positions of the start and the end nodes are written, 
    the geometry of the edge as defined in the SUMO-network is not exported.
    A map between the edge id and a segment to a numerical id would be necessary 
  """
  if outSTRM!=None:
    print("Str-Id,Sp,SegAnX,SegEnX,SegAnY,SegEnY", file=outSTRM)
  sIDm = sumolib._Running()
  for e in net._edges:
    eid = sIDm.g(e._id)
    if outSTRM!=None:
      c1 = e._from._coord
      c2 = e._to._coord
      print("%s,%s,%s,%s,%s,%s" % (eid, len(e._lanes), c1[0], c2[0], c1[1], c2[1]), file=outSTRM)
  return sIDm 

          
def fcd2fzp(inpFCD, outSTRM, further):
  """
  Reformats the contents of the given fcd-output file into a .fzp file, readable
  by PHEM. The fcd-output "fcd" must be a valid file name of an fcd-output.
  
  The "sIDm" parameter must be a map from SUMO-edge ids to their numerical 
  representation as generated by toSTR(inpNET, outSTRM).
  Returns two maps, the first from vehicle ids to a numerical representation,
  the second from vehicle type ids to a numerical representation.
  """
  sIDm = further["phemStreetMap"]
  if outSTRM!=None:
    print("t,WeltX,WeltY,Veh. No,v,Gradient,veh.Typ-Id,Str-Id", file=outSTRM)
  vIDm = sumolib._Running()
  vtIDm = sumolib._Running()
  vtIDm.g("PKW")
  vtIDm.g("LKW")
  vtIDm.g("BUS")
  for q in inpFCD:
    if q.vehicle:
      for v in q.vehicle:
        vid = vIDm.g(v.id)
        aType = _convType(v.type)
        vtid = vtIDm.g(aType)
        sid = sIDm.g(sumolib._laneID2edgeID(v.lane))
        percSlope = math.sin(float(v.slope))*100.
        if outSTRM!=None:  
          print("%s,%s,%s,%s,%s,%s,%s,%s" % (
                  sumolib._intTime(q.time), float(v.x), float(v.y), 
                  vid, float(v.speed)*3.6, percSlope, vtid, sid), file=outSTRM)
  return vIDm, vtIDm
  
  
def vehicleTypes2flt(outSTRM, vtIDm):
  """
  Currently, rather a stub than an implementation. Writes the vehicle ids stored
  in the given "vtIDm" map formatted as a .flt file readable by PHEM.

  The following may be a matter of changes:               
  - A default map is assigned to all vehicle types with the same probability 
  """
  for q in vtIDm._m:
    print("%s,%s,%s" % (vtIDm.g(q), "<VEHDIR>\PC\PC_%s.GEN" % q, 1.), file=outSTRM)




#!/usr/bin/env python
"""
@file    testUtil.py
@author  Michael Behrisch
@date    2010-10-26
@version $Id: testUtil.py 14677 2013-09-11 08:30:08Z behrisch $

This library wraps useful functions for the complex tests
and automatic GUI control
from AutoPy (http://github.com/msanders/autopy).

SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
Copyright (C) 2010-2013 DLR (http://www.dlr.de/) and contributors

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""

import os, time
_libdir = os.path.abspath(os.path.dirname(__file__))
_bindir = os.path.abspath(os.path.join(_libdir, '..', '..', 'bin'))

try:
    import autopy

    #PLAY = autopy.bitmap.Bitmap.from_string()
    PLAY = autopy.bitmap.Bitmap.open(os.path.join(_libdir, "play.png"))
    STOP = autopy.bitmap.Bitmap.open(os.path.join(_libdir, "stop.png"))

    def findAndClick(obj):
        screen = autopy.bitmap.capture_screen()
        pos = screen.find_bitmap(obj)
        autopy.mouse.move(*pos)
        autopy.mouse.click()
except ImportError:
    pass

def checkBinary(name, bindir=_bindir):
    if name == "sumo-gui":
        envName = "GUISIM_BINARY"
    else:
        envName = name.upper() + "_BINARY"
    binary = os.environ.get(envName, os.path.join(bindir, name))
    if os.name == "nt" and binary[-4:] != ".exe":
        binary += ".exe"
    if not os.path.exists(binary):
        return name
    return binary

if __name__ == "__main__":
    findAndClick(PLAY)
    time.sleep(10)
    findAndClick(STOP)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
@file    poi_alongRoads.py
@author  Ronald Nippold
@author  Daniel Krajzewicz
@author  Michael Behrisch
@date    2009-20-10
@version
@usage   poi_alongRoads.py <NET> <EDGE_ID>[,<EDGE_ID>]* <DISTANCE>

Spatial distribute of POIs along given edges on a given network.
Edges are separated with comma and without spaces in between.
The distance between POIs may be any positive real number

So far POIs are situated in the middle on all edges without regard to the type of the edge (street, junction).
 Edges may be given in arbitrary order, connected edges are found automatically
 Therefore: crossing chains of connected edges are not allowed -> this needs two different runs of this script
 Output is written in file 'pois.add.xml'

SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
Copyright (C) 2009-2013 DLR (http://www.dlr.de/) and contributors

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""

import os, sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
import sumolib.net

if len(sys.argv) < 4:
	print >> sys.stderr, "Usage: " + sys.argv[0] + " <NET> <EDGE_ID>[,<EDGE_ID>]* <DISTANCE>"
	sys.exit()
edgeList = sys.argv[2].split(',')
POIdist = float(sys.argv[3])

def findPred(connEdgesTmp,edgeList, success = 0):
	for edge in edgeList:
		if connEdgesTmp[0][1] == edge[2]:	# check first element of connEdgesTmp for predecessor
			connEdgesTmp.insert(0,edge)
			edgeList.remove(edge)
			success = 1
			return success
	return success

def findSucc(connEdgesTmp,edgeList, success = 0):
	for edge in edgeList:
		if connEdgesTmp[len(connEdgesTmp)-1][2] == edge[1]:	# check last element of connEdgesTmp for successor
			connEdgesTmp.append(edge)
			edgeList.remove(edge)
			success = 1
			return success
	return success

def poiAppend(poi_edge, poi_nr, poi_x, poi_y, poi_buf):
	poi_buf.append("\t<poi id=\"" + str(poi_edge) + "-" + str(poi_nr) + "\" type=\"default\" color=\"1,0,0\" layer=\"0\" x=\"" + str(poi_x) + "\" y=\"" + str(poi_y) + "\"/>")
	return

print("Reading net ...")
net = sumolib.net.readNet(sys.argv[1])

# find all predecessor/successor nodes for the given edges
nodeList = []
for edge in net._edges:
	for elm in sys.argv[2].split(','):
		if str(edge._id) == elm:
			data = []
			data.append(edge._id)
			data.append(edge._from._id)
			data.append(edge._to._id)
			data.append(edge._shape)
			nodeList.append(data)

# find connected edges
connEdges = []
connEdgesTmp = []
pred = 1
succ = 1
tmpList = nodeList[:]								# real copy, not just reference because of loop in nodeList
for elm in nodeList:
	if elm in tmpList:								# Has this edge already been processed?
		while pred == 1:							# search for predecessor
			if elm in tmpList:
				tmpList.remove(elm)					# don't look up the current edge in search list
			if elm not in connEdgesTmp:
				connEdgesTmp.append(elm)			# for the first element when searching for connected edges
			pred = findPred(connEdgesTmp,tmpList)
		while succ == 1:							# search for successor
			succ = findSucc(connEdgesTmp,tmpList)	# all other things already done in predecessor
		connEdges.append(connEdgesTmp)
		connEdgesTmp = []							# clean up
		pred = 1									# reset
		succ = 1

# prepare output
POIbuf = []
POIbuf.append("<?xml version=\"1.0\"?>\n")
POIbuf.append("<!--\n\nPOIs for edges:\n\t" + str(sys.argv[2]) + "\n\non network:\n\t" + str(sys.argv[1]) + "\n-->\n")
POIbuf.append("<POIs>")
# space out POIs on the found (connected) edges
for elm in connEdges:
	shapes = []										# tmp array for storing all shape coordinate of connected edges
	offset = 0										# Anything left from the edge before?
	for edge in elm:
		for p in edge[3]:
#			shapes.append(p)
			shapes.append([p[0], p[1], edge[0]])	# collect all shapes of connected edges
	for i in range(0, len(shapes)-1):
		x1 = shapes[i][0]							# point at the beginning of the current edge
		y1 = shapes[i][1]
		x2 = shapes[i+1][0]							# point at the end of the current edge
		y2 = shapes[i+1][1]
		dX = x2 - x1
		dY = y2 - y1
		eucDist = pow(pow(dX, 2) + pow(dY, 2), 0.5)	# Euclidean distance
		normAscX = dX / eucDist						# normalized Euclidean distance in x
		normAscY = dY / eucDist						# normalized Euclidean distance in y
		xCur = x1									# the current POI position
		yCur = y1
		if i == 0:
			POIid = 1
			poiAppend(shapes[i][2], POIid, xCur, yCur, POIbuf)
			POIid = POIid + 1
			POIpos = POIdist
		else:
			if shapes[i][2] != shapes[i-1][2]:		# begin of new edge
				POIid = 1
				POIpos = offset						# reset POI-position at the beginning of an edge
		# check if first (new) point is on current edge, consider possible offset from the edge before
		if eucDist + offset > POIdist:
			xCur = xCur + normAscX * (POIdist - offset)
			yCur = yCur + normAscY * (POIdist - offset)
			poiAppend(shapes[i][2], POIid, xCur, yCur, POIbuf)
			POIid = POIid + 1
			POIpos = POIpos + POIdist				# always add POIdist, reset to current offset when changing the edge
			offset = 0								# reset offset
			# other points - if possible - with normal spacing in between
			while pow(pow((x2 - xCur), 2) + pow((y2 - yCur), 2), 0.5) > POIdist:
				xCur = xCur + normAscX * POIdist
				yCur = yCur + normAscY * POIdist
				poiAppend(shapes[i][2], POIid, xCur, yCur, POIbuf)
				POIid = POIid + 1
				POIpos = POIpos + POIdist			# always add POIdist, reset to current offset when changing the edge
		# calculate offset (residual) for the next connected edge
		offset = offset + pow(pow((x2 - xCur), 2) + pow((y2 - yCur), 2), 0.5)
POIbuf.append("</POIs>")

print("writing output")
# finally write output in file
outfile = open('pois.add.xml', 'w')
for POIs in POIbuf:
	outfile.write(POIs + "\n")
outfile.close()
print("done")



#!/usr/bin/env python
"""
@file    generateTripsXml.py
@author  Yun-Pang Wang
@author  Michael Behrisch
@date    2009-02-09
@version $Id: generateTripsXml.py 14677 2013-09-11 08:30:08Z behrisch $

This script generate a trip file as input data in sumo

SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
Copyright (C) 2008-2013 DLR (http://www.dlr.de/) and contributors

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""
import os, string, sys, operator, math, datetime, random, bz2, StringIO
from xml.sax import saxutils, make_parser, handler
from optparse import OptionParser

sys.path.append(os.path.join(os.path.dirname(sys.argv[0]), "..", "assign"))
from dijkstra import dijkstraPlain
from inputs import getMatrix
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
import sumolib.net

# This class is used to build the nodes in the investigated network and 
# includes the update-function for searching the k shortest paths.

class Net(sumolib.net.Net):
    def __init__(self):
        sumolib.net.Net.__init__(self)
        self._endVertices = []
        self._startVertices = []
        
    def addNode(self, id, type=None, coord=None, incLanes=None):
        if id not in self._id2node:
            node = Vertex(id, coord, incLanes)
            self._nodes.append(node)
            self._id2node[id] = node
        self.setAdditionalNodeInfo(self._id2node[id], type, coord, incLanes)
        return self._id2node[id]

    def getEdge(self, edgeLabel):
        return self._id2edge[edgeLabel]

    def addSourceTargetEdge(self, edgeObj):
        edgeObj.source.addOutgoing(edgeObj)
        edgeObj.target.addIncoming(edgeObj)
        self._edges[edgeObj._id] = edgeObj
                          
    def getstartVertices(self):
        return self._startVertices
        
    def getendVertices(self):
        return self._endVertices
        
    def getstartCounts(self):
        return len(self._startVertices)
        
    def getendCounts(self):
        return len(self._endVertices)
        
    def getTargets(self):
        target = set()
        for end in self._endVertices:
            for sink in end.sinkConnNodes:
                if sink not in target:
                   target.add(end)
        return target
        
    def checkRoute(self, startVertex, endVertex, start, end, P, odConnTable, source, options):
        for node in endVertex.sinkConnNodes:
            length = 0.
            vertex = node
            if node in P: 
                link = P[node]
                if options.limitlength:
                    while vertex != source:
                        #if P[vertex].kind == "real":
                        length += P[vertex].length
                        vertex = P[vertex].source
                odConnTable[startVertex._id][endVertex._id].append([source.getOutgoing()[0]._id, link._id, length])
             
        if options.limitlength and len(odConnTable[startVertex._id][endVertex._id]) > 0:
            for count, item in enumerate(odConnTable[startVertex._id][endVertex._id]):
                if count == 0:
                    minLength = item[2]
                else:
                    if item[2] < minLength:
                        minLength = item[2]
            minLength *= 1.6
            for item in odConnTable[startVertex._id][endVertex._id]:
                if item[2] > minLength:
                    odConnTable[startVertex._id][endVertex._id].remove(item)

class Vertex(sumolib.net.Node):
    """
    This class is to store node attributes and the respective incoming/outgoing links.
    """
    def __init__(self, id, type=None, coord=None, incLanes=None):
        sumolib.net.Node.__init__(self, id, type, coord, incLanes)
        self.sourceConnNodes = []
        self.sinkConnNodes = []
    
    def __repr__(self):
        return self._id

# This class is uesed to store link information and estimate 
# as well as flow and capacity for the flow computation and some parameters
# read from the net.
class Edge(sumolib.net.Edge):
    """
    This class is to record link attributes
    """
    def __init__(self, id, source, target, prio, function, name):
        sumolib.net.Edge.__init__(self, id, source, target, prio, function, name)
        self.capacity = sys.maxint
        self.freeflowtime = 0.0
        self.helpacttime = 0.0
        self.weight = 0.
        self.connection = 0.
            
    def addLane(self, lane):
        sumolib.net.Edge.addLane(self, lane)
        if self._from._id == self._to._id:
            self.freeflowtime = 0.0
        else:
            self.freeflowtime = self._length / self._speed
            self.actualtime = self.freeflowtime
            self.helpacttime = self.freeflowtime

    def __repr__(self):
        cap = str(self.capacity)
        if self.capacity == sys.maxint or self.connection != 0:
            cap = "inf"
        return "%s_%s_%s_%s<%s|%s|%s|%s|%s|%s|%s|%s|%s>" % (self._function, self._id, self._from, self._to, self._speed)

class Trip:
    """
    This class is to store trip attributes.
    """
    def __init__(self, num, depart, source, sink, sourceD, sinkD):
        self.label = "%s" % num
        self.depart = depart
        self.sourceEdge = source
        self.sinkEdge = sink
        self.sourceDistrict = sourceD
        self.sinkDistrict = sinkD
        
    def __repr__(self):
        return self.label
              
# The class is for parsing the XML input file (districts). The data parsed is written into the net.
class DistrictsReader(handler.ContentHandler):
    def __init__(self, net):
        self._net = net
        self._district = None
        self.I = 100

    def startElement(self, name, attrs):
        if name == 'taz':
            self._districtSource = self._net.addNode(attrs['id'])
            self._net._startVertices.append(self._districtSource)
            self._districtSink = self._net.addNode(attrs['id'])
            self._net._endVertices.append(self._districtSink)
        elif name == 'tazSink':
            sinklink = self._net.getEdge(attrs['id'])
            self.I += 1
            conlink = self._districtSink._id + str(self.I)
            newEdge = self._net.addEdge(conlink, sinklink._to._id, self._districtSink._id, "-1", "virtual", "")
            speed = sinklink.getSpeed()
            length = sinklink.getLength()

            for i in range(0,sinklink.getLaneNumber()):
                newLane = self._net.addLane(newEdge, speed, length)
                newEdge.addLane(newLane)
                fromlane = sinklink.getLane(i)
                self._net.addConnection(sinklink, newEdge, fromlane, newLane, "s", "", -1)
            newEdge.weight = attrs['weight']
            self._districtSink.sinkConnNodes.append(sinklink._to)
            newEdge.connection = 1
        elif name == 'tazSource':
            sourcelink = self._net.getEdge(attrs['id'])
            self.I += 1
            conlink = self._districtSource._id + str(self.I)
            newEdge = self._net.addEdge(conlink, self._districtSource._id, sourcelink._from._id, "-1", "virtual", "")
            speed = sourcelink.getSpeed()
            length = sourcelink.getLength()

            for i in range(0,sourcelink.getLaneNumber()):
                newLane = self._net.addLane(newEdge, speed, length)
                newEdge.addLane(newLane)
                tolane = sourcelink.getLane(i)
                self._net.addConnection(newEdge, sourcelink, newLane, tolane, "s", "", -1)

            newEdge.weight = attrs['weight']
            self._districtSource.sourceConnNodes.append(sourcelink._from)
            newEdge.connection = 2
            
    def endElement(self, name):
        if name == 'taz':
            self._district = ''
    
def addVeh(counts, vehID, begin, period, odConnTable, startVertex, endVertex, tripList, vehIDtoODMap):
    counts += 1.
    vehID += 1
    endtime = int((float(begin + period) - 0.5) * 3600)    # The last half hour will not release any vehicles
    depart = random.randint(begin * 3600, endtime)
    if len(odConnTable[startVertex._id][endVertex._id]) > 0:
        connIndex = random.randint(0, len(odConnTable[startVertex._id][endVertex._id]) - 1)
        connPair = odConnTable[startVertex._id][endVertex._id][connIndex]
        veh = Trip(vehID, depart, connPair[0], connPair[1], startVertex._id, endVertex._id)
        vehIDtoODMap[str(vehID)] = [startVertex._id, endVertex._id]
        tripList.append(veh)
    
    return counts, vehID, tripList, vehIDtoODMap
       
def main(options):
    parser = make_parser()
    isBZ2 = False
    dataDir = options.datadir
    districts = os.path.join(dataDir, options.districtfile)
    matrix = os.path.join(dataDir, options.mtxfile)
    netfile = os.path.join(dataDir, options.netfile)
    print 'generate Trip file for:', netfile
    
    if "bz2" in netfile:
        netfile = bz2.BZ2File(netfile)
        isBZ2 = True
     
    matrixSum = 0.
    tripList = []
    net = Net()
    odConnTable = {}
    vehIDtoODMap = {}

    sumolib.net.readNet(options.netfile, net=net)

    if isBZ2:
        parser.parse(StringIO.StringIO(netfile.read()))
        netfile.close()
    else:
        parser.parse(netfile)

    parser.setContentHandler(DistrictsReader(net))
    parser.parse(districts)

    matrixPshort, startVertices, endVertices, currentMatrixSum, begin, period = getMatrix(net, options.debug, matrix, matrixSum)[:6]

    for edge in net.getEdges():
        edge.helpacttime = 0.
        
    if options.debug:
        print len(net._edges), "edges read"
        print len(net._startVertices), "start vertices read"
        print len(net._endVertices), "target vertices read"
        print 'currentMatrixSum:', currentMatrixSum
        
    if options.getconns:
        if options.debug:
            print 'generate odConnTable'
        for start, startVertex in enumerate(startVertices):
            if startVertex._id not in odConnTable:
                odConnTable[startVertex._id] = {}
                
            for source in startVertex.sourceConnNodes:
                targets = net.getTargets()
                D, P = dijkstraPlain(source, targets)
                for end, endVertex in enumerate(endVertices):
                    if startVertex._id != endVertex._id and matrixPshort[start][end] > 0.:
                        if endVertex._id not in odConnTable[startVertex._id]:
                            odConnTable[startVertex._id][endVertex._id] = []
                        net.checkRoute(startVertex, endVertex, start, end, P, odConnTable, source, options)
    else:
        if options.debug:
            print 'import and use the given odConnTable'
        sys.path.append(options.datadir)
        from odConnTables import odConnTable
        
    # output trips
    if options.verbose:
        print 'output the trip file'
    vehID = 0
    subVehID = 0
    random.seed(42)
    matrixSum = 0.
    fouttrips = file(options.tripfile, 'w')
    fouttrips.write('<?xml version="1.0"?>\n')
    print >> fouttrips, """<!-- generated on %s by $Id: generateTripsXml.py 14677 2013-09-11 08:30:08Z behrisch $ -->
    """ % datetime.datetime.now()
    fouttrips.write("<tripdefs>\n")

    if options.demandscale != 1.:
        print 'demand scale %s is used.' % options.demandscale
        for start in range(len(startVertices)):
            for end in range(len(endVertices)):
                matrixPshort[start][end] *= options.demandscale

    for start, startVertex in enumerate(startVertices):
        for end, endVertex in enumerate(endVertices):
            if startVertex._id != endVertex._id and matrixPshort[start][end] > 0.:
                counts = 0.
                if options.odestimation:
                    if matrixPshort[start][end] < 1.:
                        counts, vehID, tripList, vehIDtoODMap = addVeh(counts, vehID, begin, period, odConnTable, startVertex, endVertex, tripList, vehIDtoODMap)
                    else:
                        matrixSum += matrixPshort[start][end]
                        while (counts < float(math.ceil(matrixPshort[start][end])) and (matrixPshort[start][end] - counts) > 0.5 and float(subVehID) < matrixSum)or float(subVehID) < matrixSum:
                            counts, vehID, tripList, vehIDtoODMap = addVeh(counts, vehID, begin, period, odConnTable, startVertex, endVertex, tripList, vehIDtoODMap)
                            subVehID += 1
                else:
                    matrixSum += matrixPshort[start][end]
                    while (counts < float(math.ceil(matrixPshort[start][end])) and (matrixPshort[start][end] - counts) > 0.5 and float(vehID) < matrixSum) or float(vehID) < matrixSum:
                        counts, vehID, tripList, vehIDtoODMap = addVeh(counts, vehID, begin, period, odConnTable, startVertex, endVertex, tripList, vehIDtoODMap)
    if options.debug:
        print 'total demand:', matrixSum           
        print vehID, 'trips generated' 
    tripList.sort(key=operator.attrgetter('depart'))
    
    departpos = "free"
    if __name__ == "__main__":
        departpos = options.departpos
    for trip in tripList:
        fouttrips.write('   <trip id="%s" depart="%s" from="%s" to="%s" fromtaz="%s" totaz="%s" departlane="free" departpos="%s" departspeed="max"/>\n' \
                            % (trip.label, trip.depart, trip.sourceEdge, trip.sinkEdge, trip.sourceDistrict, trip.sinkDistrict, departpos))
    fouttrips.write("</tripdefs>")
    fouttrips.close()
    
    return odConnTable, vehIDtoODMap

if __name__ == "__main__":    
    optParser = OptionParser()
    optParser.add_option("-r", "--data-dir", dest="datadir",
                         default=os.getcwd(), help="give the data directory path")
    optParser.add_option("-n", "--net-file", dest="netfile",
                         help="define the net file (mandatory)")
    optParser.add_option("-m", "--matrix-file", dest="mtxfile",
                         help="define the matrix file (mandatory)")
    optParser.add_option("-d", "--districts-file", dest="districtfile",
                         help="define the district file (mandatory)")
    optParser.add_option("-l", "--limitlength", action="store_true", dest="limitlength",
                        default=False, help="the route length of possible connections of a given OD pair shall be less than 1.6 * min.length")   
    optParser.add_option("-t", "--trip-file", dest="tripfile",
                         default="trips.trips.xml", help="define the output trip filename")
    optParser.add_option("-x", "--odestimation", action="store_true", dest="odestimation",
                         default=False, help="generate trips for OD estimation")
    optParser.add_option("-b", "--debug", action="store_true",
                         default=False, help="debug the program")
    optParser.add_option("-v", "--verbose", action="store_true",
                         default=False, help="tell me what you are doing")
    optParser.add_option("-f", "--scale-factor", dest="demandscale", type="float", default=1., help="scale demand by ")
    optParser.add_option("-D", "--depart-pos", dest="departpos", type="choice",
                     choices=('random', 'free', 'random_free'),
                     default='free', help="choose departure position: random, free, random_free")
    optParser.add_option("-C", "--get-connections", action="store_true", dest="getconns",
                     default=True, help="generate the OD connection directory, if set as False, a odConnTables.py should be available in the defined data directory")
    (options, args) = optParser.parse_args()
    
    if not options.netfile or not options.mtxfile or not options.districtfile:
        optParser.print_help()
        sys.exit()
    
    main(options)



#!/usr/bin/python
"""
@file    removeSVN.py
@author  Daniel Krajzewicz
@author  Michael Behrisch
@date    28-08-2008
@version $Id: removeSVN.py 14677 2013-09-11 08:30:08Z behrisch $

SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
Copyright (C) 2008-2013 DLR (http://www.dlr.de/) and contributors

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""

import os, sys, stat, shutil


path = "./"
if len(sys.argv)>1:
    path = sys.argv[1]

# remove files in ".svn"
for root, dirs, files in os.walk(path):
    if root.find(".svn")>=0:
        for file in files:
            os.chmod(os.path.join(root, file), stat.S_IWRITE|stat.S_IREAD)
            os.remove(os.path.join(root, file))
        for dir in dirs:
            os.chmod(os.path.join(root, dir), stat.S_IWRITE|stat.S_IREAD)
    
# remove dirs in ".svn"
for root, dirs, files in os.walk(path):
    if ".svn" in dirs:
        dirs.remove(".svn")
        os.chmod(os.path.join(root, ".svn"), stat.S_IWRITE|stat.S_IREAD)
        shutil.rmtree(os.path.join(root, ".svn"))





"""
@file    netshiftadaptor.py
@author  Daniel Krajzewicz
@author  Michael Behrisch
@date    2008-09-01
@version $Id: netshiftadaptor.py 14953 2013-10-28 20:10:04Z behrisch $


This class performs a network reprojection
 using barycentric cordinates of two triangles
 which share the same nodes of two networks.

This means: the class is initialised with two
 networks and two lists of node ids (should be
 exactly three). The according nodes should be 
 the "same" nodes in both networks.
When "reproject" is called, all nodes' position
 of the second network are reprojected so that
 they match positions within the first network.

SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
Copyright (C) 2008-2013 DLR (http://www.dlr.de/) and contributors

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""


class NetShiftAdaptor:
    def __init__(self, net1, net2, nodes1, nodes2):
        self._net1 = net1
        self._net2 = net2
        self._nodes1 = nodes1
        self._nodes2 = nodes2
        if len(nodes1) != 3 or len(nodes2) != 3:
            raise "Both node lists must contain exactly 3 node ids"

    def reproject(self, verbose=False):
        x11 = self._net1._id2node[self._nodes1[0]]._coord[0]
        y11 = self._net1._id2node[self._nodes1[0]]._coord[1]
        x12 = self._net1._id2node[self._nodes1[1]]._coord[0]
        y12 = self._net1._id2node[self._nodes1[1]]._coord[1]
        x13 = self._net1._id2node[self._nodes1[2]]._coord[0]
        y13 = self._net1._id2node[self._nodes1[2]]._coord[1]
        x21 = self._net2._id2node[self._nodes2[0]]._coord[0]
        y21 = self._net2._id2node[self._nodes2[0]]._coord[1]
        x22 = self._net2._id2node[self._nodes2[1]]._coord[0]
        y22 = self._net2._id2node[self._nodes2[1]]._coord[1]
        x23 = self._net2._id2node[self._nodes2[2]]._coord[0]
        y23 = self._net2._id2node[self._nodes2[2]]._coord[1]
        b0 =  (x22 - x21) * (y23 - y21) - (x23 - x21) * (y22 - y21)
        for n in self._net2._nodes:
            x0 = n._coord[0]
            y0 = n._coord[1]
            b1 = ((x22 - x0)  * (y23 - y0)  - (x23 - x0)  * (y22 - y0)) / b0
            b2 = ((x23 - x0)  * (y21 - y0)  - (x21 - x0)  * (y23 - y0)) / b0
            b3 = ((x21 - x0)  * (y22 - y0)  - (x22 - x0)  * (y21 - y0)) / b0
            n._coord = (b1 * x11 + b2 * x12 + b3 * x13, b1 * y11 + b2 * y12 + b3 * y13)
        for e in self._net2._edges:
            for l in e._lanes:
                shape = []
                for p in l._shape:
                    x0 = p[0]
                    y0 = p[1]
                    b1 = ((x22 - x0)  * (y23 - y0)  - (x23 - x0)  * (y22 - y0)) / b0
                    b2 = ((x23 - x0)  * (y21 - y0)  - (x21 - x0)  * (y23 - y0)) / b0
                    b3 = ((x21 - x0)  * (y22 - y0)  - (x22 - x0)  * (y21 - y0)) / b0
                    x = (b1 * x11 + b2 * x12 + b3 * x13);
                    y = (b1 * y11 + b2 * y12 + b3 * y13);
                    shape.append( (x,y) )
                l._shape = shape
            e.rebuildShape()



"""
@file    miscutils.py
@author  Jakob.Erdmann@dlr.de
@date    2012-05-08
@version $Id: miscutils.py 15214 2013-11-25 15:17:51Z behrisch $

Common utility functions

Copyright (C) 2007-2013 DLR/FS, Germany

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

This is a duplicate of tools/util/miscutils.py from the VABENE repository
"""
import StringIO
import sys
import subprocess
import time
import os
import imp
import csv
import math
from collections import defaultdict

# append import path stanca:
#THIS_DIR == os.path.basename(__file__)
#sys.path.append(os.path.join(THIS_DIR, 'foo', 'bar'))

# http://www.python.org/dev/peps/pep-0326/
class _ExtremeType(object):

    def __init__(self, cmpr, rep):
        object.__init__(self)
        self._cmpr = cmpr
        self._rep = rep

    def __cmp__(self, other):
        if isinstance(other, self.__class__) and\
           other._cmpr == self._cmpr:
            return 0
        return self._cmpr

    def __repr__(self):
        return self._rep

uMax = _ExtremeType(1, "uMax")
uMin = _ExtremeType(-1, "uMin")

# decorator for timing a function
def benchmark(func):
    def benchmark_wrapper(*args, **kwargs):
        started = time.time()
        now = time.strftime("%a, %d %b %Y %H:%M:%S +0000", time.localtime())
        print('function %s called at %s' % (func.__name__, now))
        sys.stdout.flush()
        result = func(*args, **kwargs)
        print('function %s finished after %f seconds' % (func.__name__, time.time() - started))
        sys.stdout.flush()
        return result
    return benchmark_wrapper


class Statistics:
    def __init__(self, label=None, abs=False, histogram=False, printMin=True):
        self.label = label
        self.min = uMax
        self.min_label = None
        self.max = uMin
        self.max_label = None
        self.values = []
        self.abs = abs
        self.printMin = printMin
        if histogram:
            self.counts = defaultdict(lambda:0)
        else:
            self.counts = None

    def add(self, v, label=None):
        self.values.append(v)
        if v < self.min:
            self.min = v
            self.min_label = label
        if v > self.max:
            self.max = v
            self.max_label = label
        if self.counts is not None:
            self.counts[round(v)] += 1

    def count(self):
        return len(self.values)

    def avg(self):
        """return the mean value"""
        # XXX rename this method
        if len(self.values) > 0:
            return sum(self.values) / float(len(self.values))
        else:
            return None

    def avg_abs(self):
        """return the mean of absolute values"""
        # XXX rename this method
        if len(self.values) > 0:
            return sum(map(abs, self.values)) / float(len(self.values))
        else:
            return None

    def mean(self):
        """return the median value"""
        # XXX rename this method
        if len(self.values) > 0:
            return sorted(self.values)[len(self.values) / 2]
        else:
            return None

    def mean_abs(self):
        """return the median of absolute values"""
        # XXX rename this method
        if len(self.values) > 0:
            return sorted(map(abs,self.values))[len(self.values) / 2]
        else:
            return None

    def average_absolute_deviation_from_mean(self):
        if len(self.values) > 0:
            m = self.avg()
            return sum([abs(v - m) for v in self.values]) / len(self.values)
        else:
            return None

    def median(self):
        return self.mean()

    def median_abs(self):
        return self.mean_abs()

    def quartiles(self):
        s = sorted(self.values)
        return s[len(self.values) / 4], s[len(self.values) / 2], s[3 * len(self.values) / 4]

    def rank(self, fraction):
        if len(self.values) > 0:
            return sorted(self.values)[int(round(len(self.values) * fraction + 0.5))]
        else:
            return None

    def __str__(self):
        if len(self.values) > 0:
            min = 'min %.2f (%s), ' % (self.min, self.min_label) if self.printMin else ''
            result = '"%s": count %s, %smax %.2f (%s), mean %.2f, median %.2f' % (
                    self.label, len(self.values), min,
                    self.max, self.max_label, self.avg(), self.mean())
            if self.abs:
                result += ', mean_abs %.2f, median_abs %.2f' % (self.avg_abs(), self.mean_abs())
            if self.counts is not None:
                result += '\nhistogram: %s' % [(k,self.counts[k]) for k in sorted(self.counts.keys())]
            return result
        else:
            return '"%s": no values' % self.label


def geh(m,c):
    """Error function for hourly traffic flow measures after Geoffrey E. Havers"""
    if m+c == 0:
        return 0
    else:
        return math.sqrt(2 * (m-c) * (m-c) / (m+c))


# temporarily change working directory using 'with' statement
class working_dir:
    def __init__(self, dir):
        self.dir = dir
        self.origdir = os.getcwd()

    def __enter__(self):
        os.chdir(self.dir)

    def __exit__(self, type, value, traceback):
        os.chdir(self.origdir)




"""
@file    inputs.py
@author  Yun-Pang Wang
@author  Daniel Krajzewicz
@author  Michael Behrisch
@date    2007-10-25
@version $Id: inputs.py 14677 2013-09-11 08:30:08Z behrisch $

This script is to retrieve the assignment parameters, the OD districts and the matrix from the input files. 
Moreover, the link travel time for district connectors will be estimated.

SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
Copyright (C) 2008-2013 DLR (http://www.dlr.de/) and contributors

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""

# read the analyzed matrix
def getMatrix(net, verbose, matrix, matrixSum, demandscale=None):
    matrixPshort = []
    startVertices = []
    endVertices = []
    Pshort_EffCells = 0
    periodList = []
    if verbose:
        print 'matrix:', str(matrix)
    odpairs = 0
    origins = 0
    dest= 0
    currentMatrixSum = 0.0
    skipCount = 0
    zones = 0
    smallDemandNum = 0
    for line in open(matrix):
        if line[0] == '$':
            visumCode = line[1:3]
            if visumCode != 'VM':
                skipCount += 1
        elif line[0] != '*' and line[0] != '$':
            skipCount += 1
            if skipCount == 2:
                for elem in line.split():
                    periodList.append(float(elem))
            elif skipCount > 3:
                if zones == 0:
                    for elem in line.split():
                        zones = int(elem)
                elif len(startVertices) < zones:
                    for elem in line.split():
                        if len(elem) > 0:
                            for startVertex in net.getstartVertices():
                                if startVertex._id == elem:
                                    startVertices.append(startVertex)
                            for endVertex in net.getendVertices():
                                if endVertex._id == elem:
                                    endVertices.append(endVertex)
                    origins = len(startVertices)
                    dest = len(endVertices)
                elif len(startVertices) == zones:
                    if odpairs % origins == 0:
                        matrixPshort.append([])
                    for item in line.split():
                        elem = float(item)
                        if demandscale != None and demandscale != 1.:
                            elem *= demandscale
                        matrixPshort[-1].append(elem)
                        odpairs += 1
                        matrixSum += elem
                        currentMatrixSum += elem
                        if elem > 0.:
                            Pshort_EffCells += 1
                        if elem < 1. and elem > 0.:
                            smallDemandNum += 1
    begintime = int(periodList[0])
    assignPeriod = int(periodList[1]) - begintime
    smallDemandRatio = float(smallDemandNum)/float(Pshort_EffCells)
    
    if verbose:
        print 'Number of zones:', zones
        print 'Number of origins:', origins
        print 'Number of destinations:', dest
        print 'begintime:', begintime
        print 'currentMatrixSum:', currentMatrixSum
        print 'Effective O-D Cells:', Pshort_EffCells
        print 'number of start Vertices:', net.getstartCounts()
        print 'number of end Vertices):', net.getendCounts()
        print 'smallDemandRatio):', smallDemandRatio
    
    return matrixPshort, startVertices, endVertices, currentMatrixSum, begintime, assignPeriod, Pshort_EffCells, matrixSum, smallDemandRatio

# estimate the travel times on the district connectors
# assumption: all vehilces can reach the access links within 20 min (1200 sec) from the respective traffic zone
def getConnectionTravelTime(startVertices, endVertices):
    for vertex in startVertices:
        weightList = []
        for edge in vertex.getOutgoing():
           weightList.append(float(edge.ratio))
        if len(weightList) > 0:
            minWeight = min(weightList)

        for edge in vertex.getOutgoing():
            edge.freeflowtime = (float(edge.ratio)/minWeight) * 1200.
            edge.actualtime = edge.freeflowtime

    for vertex in endVertices:
        weightList = []
        for edge in vertex.getIncoming():
            weightList.append(float(edge.ratio))
        if len(weightList) > 0:
            minWeight = min(weightList)

        for edge in vertex.getIncoming():
            edge.freeflowtime = (float(edge.ratio)/minWeight) * 1200.
            edge.actualtime = edge.freeflowtime



#!/usr/bin/env python
"""
@file    netdiff.py
@author  Jakob Erdmann
@date    2011-10-04
@version $Id: netdiff.py 14677 2013-09-11 08:30:08Z behrisch $

Reads two networks (source, dest) and tries to produce the minimal plain-xml input
which can be loaded with netconvert alongside source to create dest

SUMO, Simulation of Urban MObility; see http://sumo-sim.org/
Copyright (C) 2011 DLR (http://www.dlr.de/) and contributors

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""

import sys
import os
import StringIO
from xml.dom import pulldom
from xml.dom import Node
from optparse import OptionParser
from subprocess import call
from collections import namedtuple, defaultdict
sys.path.append(os.path.join(os.path.dirname(sys.argv[0]), '..', 'lib'))
from testUtil import checkBinary
from OrderedMultiSet import OrderedMultiSet

INDENT = 4

# file types to compare
TYPE_NODES       = '.nod.xml'
TYPE_EDGES       = '.edg.xml'
TYPE_CONNECTIONS = '.con.xml'
TYPE_TLLOGICS    = '.tll.xml'
PLAIN_TYPES = [
        TYPE_NODES,
        TYPE_EDGES,
        TYPE_CONNECTIONS,
        TYPE_TLLOGICS
        ] 

# traffic lights have some peculiarities
# CAVEAT1 - ids are not unique (only in combination with programID)
# CAVEAT2 - the order of their children (phases) is important.
#     this makes partial diffs unfeasible. The easiest solution is to forgo diffs and always export the whole new traffic light
# CAVEAT3 - deletes need not be written because they are also signaled by a changed node type 
#     (and they complicate the handling of deleted tl-connections)
# CAVEAT4 - deleted connections must be written with their tlID and tlIndex, otherwise
#     parsing in netconvert becomes tedious
# CAVEAT5 - phases must maintain their order
# CAVEAT6 - identical phases may occur multiple times, thus OrderedMultiSet

TAG_TLL = 'tlLogic'
TAG_CONNECTION = 'connection'

# see CAVEAT1
IDATTRS = defaultdict(lambda: ('id',))
IDATTRS[TAG_TLL] = ('id', 'programID')
IDATTRS[TAG_CONNECTION] = ('from', 'to', 'fromLane', 'toLane')
IDATTRS['interval'] = ('begin', 'end')

DELETE_ELEMENT = 'delete' # the xml element for signifying deletes

# provide an order for the attribute names
ATTRIBUTE_NAMES = {
        #'.nod.xml' : ()
        #'.edg.xml' : () 
        #'.con.xml' : ()
        }

# default values for the given attribute (needed when attributes appear in source but do not appear in dest)
DEFAULT_VALUES = defaultdict(lambda: "")
DEFAULT_VALUES['width'] = "-1"
DEFAULT_VALUES['offset'] = "0"
RESET = 0


# stores attributes for later comparison
class AttributeStore:

    def __init__(self, type, copy_tags, level=1):
        # xml type being parsed
        self.type = type
        # tag names to copy even if unchanged
        self.copy_tags = copy_tags
        # indent level
        self.level = level
        # dict of names-tuples
        self.attrnames = {}
        # sets of (tag, id) preserve order to avoid dangling references during loading
        self.ids_deleted = OrderedMultiSet()
        self.ids_created = OrderedMultiSet()
        self.ids_copied = OrderedMultiSet()
        # dict from (tag, id) to (names, values, children)
        self.id_attrs = {}
        # dict from tag to (names, values)-sets, need to preserve order (CAVEAT5)
        self.idless_deleted = defaultdict(lambda:OrderedMultiSet())
        self.idless_created = defaultdict(lambda:OrderedMultiSet())
        self.idless_copied = defaultdict(lambda:OrderedMultiSet())


    # getAttribute returns "" if not present
    def getValue(self, node, name):
        if node.hasAttribute(name):
            return node.getAttribute(name)
        else:
            return None


    def getNames(self, xmlnode):
        idattrs = IDATTRS[xmlnode.localName]
        a = xmlnode.attributes
        all = [a.item(i).localName for i in range(a.length)]
        instance = tuple([n for n in all if n not in idattrs])
        if not instance in self.attrnames:
            self.attrnames[instance] = instance
        # only store a single instance of this tuple to conserve memory
        return self.attrnames[instance]


    def getAttrs(self, xmlnode):
        names = self.getNames(xmlnode)
        values = tuple([self.getValue(xmlnode, a) for a in names])
        children = None
        if any([c.nodeType == Node.ELEMENT_NODE for c in xmlnode.childNodes]):
            children = AttributeStore(self.type, self.copy_tags, self.level + 1)
        tag = xmlnode.localName
        id = tuple([xmlnode.getAttribute(a) for a in IDATTRS[tag] if xmlnode.hasAttribute(a)])
        return tag, id, children, (names, values, children)


    def store(self, xmlnode):
        tag, id, children, attrs = self.getAttrs(xmlnode)
        tagid = (tag, id)
        if id != ():
            self.ids_deleted.add(tagid)
            self.id_attrs[tagid] = attrs
            if children:
                for child in xmlnode.childNodes:
                    if child.nodeType == Node.ELEMENT_NODE:
                        children.store(child)
        else:
            self.no_children_supported(children, tag)
            self.idless_deleted[tag].add(attrs)


    def compare(self, xmlnode):
        tag, id, children, attrs = self.getAttrs(xmlnode)
        tagid = (tag, id)
        if id != ():
            if tagid in self.ids_deleted:
                self.ids_deleted.remove(tagid)
                self.id_attrs[tagid] = self.compareAttrs(self.id_attrs[tagid], attrs, tag)
            else:
                self.ids_created.add(tagid)
                self.id_attrs[tagid] = attrs

            children = self.id_attrs[tagid][2]
            if children:
                for child in xmlnode.childNodes:
                    if child.nodeType == Node.ELEMENT_NODE:
                        children.compare(child)
                if tag == TAG_TLL or tag in self.copy_tags: # see CAVEAT2
                    child_strings = StringIO.StringIO()
                    children.writeDeleted(child_strings)
                    children.writeCreated(child_strings)
                    children.writeChanged(child_strings)
                    if child_strings.len > 0 or tag in self.copy_tags:
                        # there are some changes. Go back and store everything
                        children = AttributeStore(self.type, self.copy_tags, self.level + 1)
                        for child in xmlnode.childNodes:
                            if child.nodeType == Node.ELEMENT_NODE:
                                children.compare(child)
                        self.id_attrs[tagid] = self.id_attrs[tagid][0:2] + (children,)


        else:
            self.no_children_supported(children, tag)
            if attrs in self.idless_deleted[tag]:
                self.idless_deleted[tag].remove(attrs)
                if tag in self.copy_tags:
                    self.idless_copied[tag].add(attrs)
            else:
                self.idless_created[tag].add(attrs)


    def no_children_supported(self, children, tag):
        if children:
            print("WARNING: Handling of children only supported for elements with id. Ignored for element '%s'" % tag)


    def compareAttrs(self, sourceAttrs, destAttrs, tag):
        snames, svalues, schildren = sourceAttrs
        dnames, dvalues, dchildren = destAttrs
        # for traffic lights, always use dchildren
        if schildren and dchildren:
            dchildren = schildren
        if snames == dnames:
            values = tuple([self.diff(n,s,d) for n,s,d in zip (snames,svalues,dvalues)])
            return snames, values, dchildren
        else:
            sdict = defaultdict(lambda:None, zip(snames, svalues))
            ddict = defaultdict(lambda:None, zip(dnames, dvalues))
            names = tuple(set(snames + dnames))
            values = tuple([self.diff(n,sdict[n],ddict[n]) for n in names])
            return names, values, dchildren


    def diff(self, name, sourceValue, destValue):
        if sourceValue == destValue:
            return None
        elif destValue == None:
            return DEFAULT_VALUES[name]
        else:
            return destValue


    def writeDeleted(self, file):
        # data loss if two elements with different tags 
        # have the same id
        for tag, id in self.ids_deleted:
            additional = ""
            if self.type == TYPE_TLLOGICS and tag == TAG_CONNECTION:
                # see CAVEAT4
                names, values, children = self.id_attrs[(tag, id)]
                additional = " " + self.attr_string(names, values)

            comment_start, comment_end = ("", "")
            if tag == TAG_TLL: # see CAVEAT3
                comment_start, comment_end = ("<!-- implicit via changed node type: ", " -->")
            self.write(file, '%s<%s %s%s/>%s\n' % (
                comment_start,
                DELETE_ELEMENT, self.id_string(tag, id), additional,
                comment_end))
        # data loss if two elements with different tags 
        # have the same list of attributes and values
        for value_set in self.idless_deleted.itervalues():
            self.write_idless(file, value_set, DELETE_ELEMENT)


    def writeCreated(self, file):
        self.write_tagids(file, self.ids_created, True)
        for tag, value_set in self.idless_created.iteritems():
            self.write_idless(file, value_set, tag)


    def writeChanged(self, file):
        tagids_changed = OrderedMultiSet(self.id_attrs.keys()) - (self.ids_deleted | self.ids_created)
        self.write_tagids(file, tagids_changed, False)


    def writeCopies(self, file, copy_tags):
        tagids_unchanged = OrderedMultiSet(self.id_attrs.keys()) - (self.ids_deleted | self.ids_created)
        self.write_tagids(file, tagids_unchanged, False)
        for tag, value_set in self.idless_copied.iteritems():
            self.write_idless(file, value_set, tag)


    def write_idless(self, file, attr_set, tag):
        for names, values, children in attr_set:
            self.write(file, '<%s %s/>\n' % (tag, self.attr_string(names, values)))


    def write_tagids(self, file, tagids, create):
        for tagid in tagids:
            tag, id = tagid
            names, values, children = self.id_attrs[tagid]
            attrs = self.attr_string(names, values)
            child_strings = StringIO.StringIO()
            if children:
                # writeDeleted is not supported
                children.writeCreated(child_strings)
                children.writeChanged(child_strings)

            if len(attrs) > 0 or child_strings.len > 0 or create or tag in self.copy_tags:
                close_tag = "/>\n"
                if child_strings.len > 0:
                    close_tag = ">\n%s" % child_strings.getvalue()
                self.write(file, '<%s %s %s%s' % (
                    tag,
                    self.id_string(tag, id),
                    attrs,
                    close_tag))
                if child_strings.len > 0:
                    self.write(file, "</%s>\n" % tag)


    def write(self, file, item):
        file.write(" " * INDENT * self.level)
        file.write(item)


    def attr_string(self, names, values):
        return ' '.join(['%s="%s"' % (n,v) for n,v in zip(names, values) if v != None])

    def id_string(self, tag, id):
        idattrs = IDATTRS[tag]
        return ' '.join(['%s="%s"' % (n,v) for n,v in zip(idattrs, id)])


def parse_args():
    USAGE = "Usage: " + sys.argv[0] + " <source> <dest> <output-prefix>"
    optParser = OptionParser()
    optParser.add_option("-v", "--verbose", action="store_true",
            default=False, help="Give more output")
    optParser.add_option("-p", "--use-prefix", action="store_true",
            default=False, help="interpret source and dest as plain-xml prefix instead of network names")
    optParser.add_option("-d", "--direct", action="store_true",
            default=False, help="compare source and dest files directly")
    optParser.add_option("-c", "--copy", help="comma-separated list of element names to copy (if they are unchanged)")
    optParser.add_option("--path", dest="path",
            default=os.environ.get("SUMO_BINDIR", ""), help="Path to binaries")
    options, args = optParser.parse_args()
    if len(args) != 3:
        sys.exit(USAGE)
    if options.use_prefix and options.direct:
        optParser.error("Options --use-prefix and --direct are mutually exclusive")
    options.source, options.dest, options.outprefix = args
    return options 


def create_plain(netfile, netconvert):
    prefix = netfile[:-8]
    call([netconvert, 
        "--sumo-net-file", netfile, 
        "--plain-output-prefix", prefix])
    return prefix


# creates diff of a flat xml structure 
# (only children of the root element and their attrs are compared)
def xmldiff(source, dest, diff, type, copy_tags):
    attributeStore = AttributeStore(type, copy_tags)
    root_open = None
    have_source = os.path.isfile(source)
    have_dest = os.path.isfile(dest)
    if have_source:
        root_open, root_close = handle_children(source, attributeStore.store)
    if have_dest:
        root_open, root_close = handle_children(dest, attributeStore.compare)

    if not have_source and not have_dest:
        print "Skipping %s due to lack of input files" % diff
    else:
        if not have_source:
            print "Source file %s is missing. Assuming all elements are created" % source
        elif not have_dest:
            print "Dest file %s is missing. Assuming all elements are deleted" % dest

        with open(diff, 'w') as diff_file:
            diff_file.write(root_open)
            if copy_tags:
                attributeStore.write(diff_file, "<!-- Copied Elements -->\n")
                attributeStore.writeCopies(diff_file, copy_tags)
            attributeStore.write(diff_file, "<!-- Deleted Elements -->\n")
            attributeStore.writeDeleted(diff_file)
            attributeStore.write(diff_file, "<!-- Created Elements -->\n")
            attributeStore.writeCreated(diff_file)
            attributeStore.write(diff_file, "<!-- Changed Elements -->\n")
            attributeStore.writeChanged(diff_file)
            diff_file.write(root_close)


# calls function handle_parsenode for all children of the root element
# returns opening and closing tag of the root element
def handle_children(xmlfile, handle_parsenode):
    root_open = None
    root_close = None
    level = 0
    xml_doc = pulldom.parse(xmlfile)
    for event, parsenode in xml_doc:
        if event == pulldom.START_ELEMENT: 
            # print level, parsenode.getAttribute(ID_ATTR)
            if level == 0:
                root_open = parsenode.toprettyxml(indent="")
                # since we did not expand root_open contains the closing slash
                root_open = root_open[:-3] + ">\n"
                root_close = "</%s>\n" % parsenode.localName
            if level == 1:
                xml_doc.expandNode(parsenode) # consumes END_ELEMENT, no level increase
                handle_parsenode(parsenode)
            else:
                level += 1
        elif event == pulldom.END_ELEMENT: 
            level -= 1
    return root_open, root_close


# run
def main():
    options = parse_args()
    copy_tags = options.copy.split(',') if options.copy else []
    if options.direct:
        type = '.xml'
        xmldiff(options.source, 
                options.dest, 
                options.outprefix + type, 
                type,
                copy_tags)
    else:
        if not options.use_prefix:
            netconvert = checkBinary("netconvert", options.path)        
            options.source = create_plain(options.source, netconvert)
            options.dest = create_plain(options.dest, netconvert)
        for type in PLAIN_TYPES:
            xmldiff(options.source + type, 
                    options.dest + type, 
                    options.outprefix + type, 
                    type,
                    copy_tags)

if __name__ == "__main__":
    main()



from pylab import *
from ConfigParser import *

def compareData(name1, name2, str):
    filename = name1 + '.ini'
    ini = ConfigParser()
    ini.read(filename)
    
    distrWE = ini.get("demand", "distrWE")
    distr = ("poisson" if (distrWE=='p') else "uniform")
    
    vehphWEA = eval(ini.get("demand", "vehphWEA"))
    vehphNSA = eval(ini.get("demand", "vehphNSA"))
	
    filename1 = name1 + ".dat"
    
    data1 = load(filename1)
    
    flowWEA1 = data1[:,0]
    flowNSA1 = data1[:,1]
    qWEA1 = data1[:,2]
    qNSA1 = data1[:,3]
    avgDelayWEA1 = data1[:,4]
    avgDelayNSA1 = data1[:,5]
    avgDelayA1 = data1[:,6]
    greenWEA1 = data1[:,7]
    greenNSA1 = data1[:,8]
    
    filename2 = name2 + ".dat"
    
    data2 = load(filename2)
    
    flowWEA2 = data2[:,0]
    flowNSA2 = data2[:,1]
    qWEA2 = data2[:,2]
    qNSA2 = data2[:,3]
    avgDelayWEA2 = data2[:,4]
    avgDelayNSA2 = data2[:,5]
    avgDelayA2 = data2[:,6]
    greenWEA2 = data2[:,7]
    greenNSA2 = data2[:,8]    
    
    [X,Y] = meshgrid(vehphWEA, vehphNSA)
    
    figure(figsize=(6,6))

    Z1 = array(griddata(flowWEA1, flowNSA1, avgDelayA1, X, Y))
    Z2 = array(griddata(flowWEA2, flowNSA2, avgDelayA2, X, Y))
    
    Z = Z1 / Z2
    
    subplot(1,1,1)
    contourf(X,Y,Z,concatenate((frange(0,2,0.1),frange(2,3,1))))
    xlabel('Input flow WE')
    ylabel('Input flow NS')
    colorbar()

    title('Average Delay VA vs. FC (ratio), ' + distr)
    savefig(str + ".png")
    
        
    Z1 = array(griddata(flowWEA1, flowNSA1, avgDelayWEA1, X, Y))
    Z2 = array(griddata(flowWEA2, flowNSA2, avgDelayWEA2, X, Y))
    
   
    
    Z = Z1 / Z2

    figure(figsize=(12,6))
    subplot(1,2,1)
    contourf(X,Y,Z,concatenate((frange(0,2,0.1),frange(2,3,1))))
    xlabel('Input flow WE')
    ylabel('Input flow NS')
    title('Average Delay WE')
    colorbar()
 
 
    Z1 = array(griddata(flowWEA1, flowNSA1, avgDelayNSA1, X, Y))
    Z2 = array(griddata(flowWEA2, flowNSA2, avgDelayNSA2, X, Y))
    
    
    Z = Z1 / Z2
 
    subplot(1,2,2)
    contourf(X,Y,Z,concatenate((frange(0,2,0.1),frange(2,3,1))))
    xlabel('Input flow WE')
    ylabel('Input flow NS')
    title('Average Delay NS')
    colorbar()   

    suptitle('Average Delay VA vs. FC (ratio), ' + distr)
    savefig(str + "WENS.png")

def plotDiagram(name):
    filename = name + ".dat"
    
    X = load(filename)

    flowWEA = X[:,0]
    flowNSA = X[:,1]
    qWEA = X[:,2]
    qNSA = X[:,3]
    avgDelayWEA = X[:,4]
    avgDelayNSA = X[:,5]
    avgDelayA = X[:,6]
    greenWEA = X[:,7]
    greenNSA = X[:,8]



    filename = name + '.ini'
    ini = ConfigParser()
    ini.read(filename)
    
    N = ini.getint("general", "N")
    cut = ini.getboolean("general", "cut")
    
    distrWE = ini.get("demand", "distrWE")
    distrNS = ini.get("demand", "distrNS")

    tlType = ini.get("TL", "tlType")

    vehphWEA = eval(ini.get("demand", "vehphWEA"))
    vehphNSA = eval(ini.get("demand", "vehphNSA"))
    

    maxDelay = max(max(avgDelayWEA), max(avgDelayNSA))
 
    maxD = max(avgDelayA)
    c = maxD / (exp(1)-1)
    V = [c*(exp(i)-1) for i in frange(0,1,0.01)]
 
    [X,Y] = meshgrid(vehphWEA, vehphNSA)
    

    Z = griddata(flowWEA, flowNSA, avgDelayWEA, X, Y)
    
   
    figure(figsize=(12,12))
    subplot(2,2,1)
    #contour(X, Y, Z, 100)
    contourf(X,Y,Z,range(0,180,5))
    
    colorbar()
    title('Average Delay WE')
    xlabel('Input flow WE')
    ylabel('Input flow NS')

    Z = griddata(flowWEA, flowNSA, avgDelayNSA, X, Y)

    subplot(2,2,2)
    #contour(X, Y, Z, 100)
    contourf(X,Y,Z,range(0,180,5))
    colorbar()
    title('Average Delay NS')
    xlabel('Input flow WE')
    ylabel('Input flow NS')
    

    Z = griddata(qWEA, qNSA, avgDelayWEA, X, Y)
    
    avgDelayWECut = [Z[i][i] for i in range(len(Z))]

    subplot(2,2,3)
    #contour(X, Y, Z, 100)
    contourf(X,Y,Z,range(0,180,5))
    colorbar()
    title('Average Delay WE')
    xlabel('Flow q WE')
    ylabel('Flow q NS')

    Z = griddata(qWEA, qNSA, avgDelayNSA, X, Y)
    
    avgDelayNSCut = [Z[i][i] for i in range(len(Z))]

    subplot(2,2,4)
    #contour(X, Y, Z, 100)
    contourf(X,Y,Z,range(0,180,5))
    colorbar()
    title('Average Delay NS')
    xlabel('Flow q WE')
    ylabel('Flow q NS')
    
    suptitle("Average Delay (WE " + ("poisson" if distrWE == 'p' else "uniform") + ", NS " + ("poisson" if distrNS == 'p' else "uniform") + ", " + tlType + ")")
    
    savefig(name + "AvgDelayNSWE.png")
    close()    
    
    
    
    Z = griddata(flowWEA, flowNSA, avgDelayA, X, Y)
    
    figure(figsize=(12,6))
    subplot(1,2,1)
    #contour(X, Y, Z, 100)
    contourf(X,Y,Z,range(0,180,5))
    colorbar()
    title('Average Delay')
    xlabel('Input flow WE')
    ylabel('Input flow NS')
    
    Z = griddata(qWEA, qNSA, avgDelayA, X, Y)
    
    avgDelayCut = [Z[i][i] for i in range(len(Z))]
    
    subplot(1,2,2)
    #contour(X, Y, Z, 100)
    contourf(X,Y,Z,range(0,180,5))
#    clim(0, maxDelay)
    colorbar()
    title('Average Delay')
    xlabel('Flow q WE')
    ylabel('Flow q NS')
    
    suptitle("Average Delay (WE " + ("poisson" if distrWE == 'p' else "uniform") + ", NS " + ("poisson" if distrNS == 'p' else "uniform") + ", " + tlType + ")")
    
    savefig(name + "AvgDelay.png")
    close()
    
    
    
    Z = griddata(flowWEA, flowNSA, qWEA, X, Y)
    
    figure(figsize=(12,6))
    subplot(1,2,1)
    contourf(X, Y, Z, range(0,1250,50))
    colorbar()
    title('Flow q WE')
    xlabel('Input flow WE')
    ylabel('Input flow NS')
    
    Z = griddata(flowWEA, flowNSA, qNSA, X, Y)
    
    subplot(1,2,2)
    contourf(X, Y, Z, range(0,1250,50))
    colorbar()
    title('Flow q NS')
    xlabel('Input flow WE')
    ylabel('Input flow NS')

    suptitle("Input flow vs. flow q (WE " + ("poisson" if distrWE == 'p' else "uniform") + ", NS " + ("poisson" if distrNS == 'p' else "uniform") + ", " + tlType + ")")    
    
    savefig(name + "flowvsq.png")
   
    close()
    
    
    maxGreen = max(max(greenWEA), max(greenNSA))
    
    Z = griddata(flowWEA, flowNSA, greenWEA, X, Y)
    
    figure(figsize=(12,12))
    subplot(2,2,1)
    contourf(X, Y, Z, range(0,100,5))
 #   clim(0, maxGreen)
    colorbar()
    title('Green-time WE')
    xlabel('Input flow WE')
    ylabel('Input flow NS')
    
    Z = griddata(flowWEA, flowNSA, greenNSA, X, Y)
    
    subplot(2,2,2)
    contourf(X, Y, Z, range(0,100,5))
#    clim(0, maxGreen)
    colorbar()
    title('Green-time NS')
    xlabel('Input flow WE')
    ylabel('Input flow NS')
    
    Z = griddata(qWEA, qNSA, greenWEA, X, Y)
    
    greenWECut = [Z[i][i] for i in range(len(Z))]
    
    subplot(2,2,3)
    contourf(X, Y, Z, range(0,100,5))
 #   clim(0, maxGreen)
    colorbar()
    title('Green-time WE')
    xlabel('Flow q WE')
    ylabel('Flow q NS')
    
    Z = griddata(qWEA, qNSA, greenNSA, X, Y)
    
    greenNSCut = [Z[i][i] for i in range(len(Z))]
    
    subplot(2,2,4)
    contourf(X, Y, Z, range(0,100,5))
    colorbar()
    title('Green-time NS')
    xlabel('Flow q WE')
    ylabel('Flow q NS')

    suptitle("Green-time (WE " + ("poisson" if distrWE == 'p' else "uniform") + ", NS " + ("poisson" if distrNS == 'p' else "uniform") + ", " + tlType + ")")
    
    savefig(name + "GreenTime.png")
    close()    
    
    
    q = X[0]
    
    figure()
    title("CUT  (WE " + ("poisson" if distrWE == 'p' else "uniform") + ", NS " + ("poisson" if distrNS == 'p' else "uniform") + ", " + tlType + ")")
    xlabel('Flow q')
    plot(q[0:15], avgDelayWECut[0:15],q[0:15], avgDelayNSCut[0:15], q[0:15], avgDelayCut[0:15],q[0:15], greenWECut[0:15], q[0:15], greenNSCut[0:15])
    legend(('avgDelayWE', 'avgDelayNS', 'avgDelay','greenWE', 'greenNS'),loc='upper left')
    savefig(name + "CUT.png")
    close()  